using System;
using System.Collections.Generic;
using File=System.IO.File;

namespace MorrowindModManager {
    public static class Conflicts {

        public static void UpdateConflicts() {
            foreach(mmod o in Program.Data.mmods) UpdateConflict(o);
        }
        public static void UpdateConflict(mmod o) {
            if(o.Conflict==ConflictLevel.Active) return;
            o.Conflict=ConflictLevel.NoConflict;
            //Check that no esps already exist
            foreach(string s in o.AllPlugins) {
                if(File.Exists("data files\\"+s)) {
                    o.Conflict=ConflictLevel.Unusable;
                    return;
                }
            }
            //Check for any script defined conflicting mods
            foreach(ConflictData cd in o.ConflictsWith) {
                if(Program.Data.DoesModExist(cd, true)) {
                    if((int)cd.level>(int)o.Conflict) o.Conflict=cd.level;
                }
            }
            foreach(mmod o2 in Program.Data.mmods) {
                if(o2.Conflict!=ConflictLevel.Active||o2==o) continue;
                foreach(ConflictData cd in o2.ConflictsWith) {
                    if(cd==o) {
                        if((int)cd.level>(int)o.Conflict) o.Conflict=cd.level;
                    }
                }
            }
            if(o.Conflict==ConflictLevel.MajorConflict) return;
            //Check that no data files already exist
            foreach(DataFileInfo df in o.AllDataFiles) {
                DataFileInfo dfi=Program.Data.GetDataFile(df);
                if(dfi==null) {
                    if(!File.Exists("data files\\"+df.FileName)) continue;
                    o.Conflict=ConflictLevel.MajorConflict;
                    return;
                } else if(df.CRC==dfi.CRC) {
                    o.Conflict=ConflictLevel.MinorConflict;
                } else {
                    o.Conflict=ConflictLevel.MajorConflict;
                    return;
                }
            }
        }
        public static string ConflictReport(mmod o) {
            string epilog=Environment.NewLine+Environment.NewLine+"This conflict report displays only data file and script defined conflicts"+
                Environment.NewLine+"For EDID conflicts, please use the full conflict report tool.";
            if(o.Conflict==ConflictLevel.Active) return "["+o.FileName+"] (Active)"+Environment.NewLine+
                "Data file and script conflicts are not tracked for active mods."+epilog;
            string ModReport="";
            ConflictLevel Conflict=ConflictLevel.NoConflict;
            //Check that no esps already exist
            foreach(string s in o.AllPlugins) {
                EspInfo ei=Program.Data.GetEsp(s);
                if(ei!=null||File.Exists("data files\\"+s)) {
                    Conflict=ConflictLevel.Unusable;
                    ModReport+="Plugin file "+s+" already exists."+Environment.NewLine;
                    if(ei!=null) {
                        ModReport+="- belongs to: "+ei.BelongsTo;
                    } else {
                        ModReport+="- esp missing from mwmm database";
                    }
                    ModReport+=Environment.NewLine;
                }
            }
            //Check for any script defined conflicting mods
            foreach(ConflictData cd in o.ConflictsWith) {
                if(Program.Data.DoesModExist(cd, true)) {
                    if((int)cd.level>(int)Conflict) Conflict=cd.level;
                    ModReport+="Conflicts with active mod "+cd.File+Environment.NewLine;
                    if(cd.Comment!=null) ModReport+="- "+cd.Comment+Environment.NewLine;
                }
            }
            foreach(mmod o2 in Program.Data.mmods) {
                if(o2.Conflict!=ConflictLevel.Active||o2==o) continue;
                foreach(ConflictData cd in o2.ConflictsWith) {
                    if(cd==o) {
                        if((int)cd.level>(int)Conflict) Conflict=cd.level;
                        ModReport+="Active mod "+o2.FileName+" reports that it conflicts with this one"+Environment.NewLine;
                        if(cd.Comment!=null) ModReport+="- "+cd.Comment+Environment.NewLine;
                    }
                }
            }
            //Check that no data files already exist
            foreach(DataFileInfo df in o.AllDataFiles) {
                DataFileInfo dfi=Program.Data.GetDataFile(df);
                if(dfi==null) {
                    if(!File.Exists("data files\\"+df.FileName)) continue;
                    if((int)ConflictLevel.MajorConflict>(int)Conflict) Conflict=ConflictLevel.MajorConflict;
                    ModReport+="Data file "+df.FileName+" already exists"+Environment.NewLine;
                    ModReport+="- No data found on file."+Environment.NewLine;
                } else if(df.CRC==dfi.CRC) {
                    if((int)ConflictLevel.MinorConflict>(int)Conflict) Conflict=ConflictLevel.MinorConflict;
                    ModReport+="Data file "+df.FileName+" already exists"+Environment.NewLine;
                    ModReport+="- CRC's match, so probably nothing to worry about."+Environment.NewLine;
                    ModReport+="- Data file owned by "+dfi.Owners+Environment.NewLine;
                } else {
                    if((int)ConflictLevel.MajorConflict>(int)Conflict) Conflict=ConflictLevel.MajorConflict;
                    ModReport+="Data file "+df.FileName+" already exists"+Environment.NewLine;
                    ModReport+="- CRC mismatch. The new file is different from the old."+Environment.NewLine;
                    ModReport+="- Data file owned by "+dfi.Owners+Environment.NewLine;
                }
            }
            string prolog="["+o.FileName+"] (";
            switch(Conflict) {
            case ConflictLevel.Unusable: prolog+="unusable"; break;
            case ConflictLevel.MajorConflict: prolog+="major conflict"; break;
            case ConflictLevel.MinorConflict: prolog+="minor conflict"; break;
            case ConflictLevel.NoConflict: prolog+="no conflicts"; break;
            }
            prolog+=")"+Environment.NewLine;
            return prolog+ModReport+epilog;
        }
    }
}
