using System;
using System.IO;
using System.Collections.Generic;
using Formatter=System.Runtime.Serialization.Formatters.Binary.BinaryFormatter;

namespace MorrowindModManager {
    public static class MorrowindESP {

        public static bool[] ArePluginsActive(string[] fileNames) {
            string[] FileNames=new string[fileNames.Length];
            bool[] result=new bool[fileNames.Length];
            for(int i=0;i<result.Length;i++) {
                FileNames[i]=fileNames[i].ToLower();
                result[i]=false;
            }
            string[] ActiveEsps=MorrowindINI.GetCleanINISection("[Game Files]");
            int found;
            foreach(string s in ActiveEsps) {
                if(s==""||s.StartsWith("#")) continue;
                if((found=Array.IndexOf(FileNames, s.ToLower()))!=-1) result[found]=true;
            }
            return result;
        }

        public static void SetActivePlugins() {
            List<string> plugins=new List<string>();
            int count=0;
            foreach(EspInfo ei in Program.Data.Esps) {
                if(ei.Active) {
                    plugins.Add("GameFile"+count+"="+ei.FileName);
                    count++;
                }
            }
            MorrowindINI.ReplaceINISection("[Game Files]",plugins.ToArray());
        }
    }

    public static class MorrowindINI {
        private static readonly string ini=Program.morrowindINIDir+"morrowind.ini";

        public static bool CreateINI() {
            if(!Directory.Exists(Program.morrowindINIDir)) return false;
            if(File.Exists(ini)) return true;
            try { File.Copy("morrowind_default.ini", ini); } catch { }
            return File.Exists(ini);
        }

        public static string GetINIValue(string section, string name) {
            string[] ss=GetINISection(section);
            if(ss==null) throw new mwmmException("morrowind.ini section "+section+" does not exist");
            name=name.ToLower();
            foreach(string s in ss) {
                if(s.Trim().ToLower().StartsWith(name+"=")) {
                    string res=s.Substring(s.IndexOf('=')+1).Trim();
                    int i=res.IndexOf(';');
                    if(i!=-1) res=res.Substring(0, i-1);
                    return res;
                }
            }
            return null;
        }

        public static void WriteINIValue(string section,string name,string value) {
            List<string> ss=new List<string>(GetINISection(section));
            if(ss==null) throw new mwmmException("morrowind.ini section "+section+" does not exist");
            bool matched=false;
            string lname=name.ToLower();
            for(int i=0;i<ss.Count;i++) {
                string s=ss[i];
                if(s.Trim().ToLower().StartsWith(lname+"=")) {
                    if(value==null) {
                        ss.RemoveAt(i--);
                    } else {
                        ss[i]=name+"="+value;
                     
                    }
                    matched=true;
                    break;
                }
            }
            if(!matched) ss.Add(name+"="+value);
            ReplaceINISection(section,ss.ToArray());
        }

        public static string[] GetCleanINISection(string section) {
            List<string> sec=new List<string>(GetINISection(section));
            for(int i=0;i<sec.Count;i++) {
                int j=sec[i].IndexOf(';');
                if(j!=-1) sec[i]=sec[i].Remove(j);
                j=sec[i].IndexOf('=');
                if(j!=-1) sec[i]=sec[i].Substring(j+1);
                sec[i]=sec[i].Trim();
                if(sec[i]=="") sec.RemoveAt(i--);
            }
            return sec.ToArray();
        }

        private static string[] GetINISection(string section) {
            StreamReader sr=new StreamReader(File.OpenRead(ini), System.Text.Encoding.Default);
            List<string> contents=new List<string>();
            bool InSection=false;
            section=section.ToLower();
            while(sr.Peek()!=-1) {
                string s=sr.ReadLine();
                if(InSection) {
                    if(s.Trim().StartsWith("[")&&s.Trim().EndsWith("]")) break;
                    contents.Add(s);
                } else {
                    if(s.Trim().ToLower()==section) InSection=true;
                }
            }
            sr.Close();
            if(!InSection) return null;
            return contents.ToArray();
        }

        public static void ReplaceINISection(string section, string[] ReplaceWith) {
            StreamReader sr=new StreamReader(File.OpenRead(ini), System.Text.Encoding.Default);
            List<string> contents=new List<string>();
            section=section.ToLower();
            bool InSection=false;
            while(sr.Peek()!=-1) {
                string s=sr.ReadLine();
                if(!InSection) {
                    contents.Add(s);
                    if(s.Trim().ToLower()==section) {
                        InSection=true;
                        contents.AddRange(ReplaceWith);
                    }
                } else {
                    if(s.Trim().StartsWith("[")&&s.Trim().EndsWith("]")) {
                        contents.Add(s);
                        InSection=false;
                    }
                }
            }
            sr.Close();
            StreamWriter sw=new StreamWriter(File.Create(ini), System.Text.Encoding.Default);
            foreach(string s in contents) {
                sw.WriteLine(s);
            }
            sw.Close();
        }

        public static bool IsBSARegistered(string FileName) {
            FileName=FileName.ToLower();
            string[] BSAs=GetINISection("[Archives]");
            for(int i=0;i<BSAs.Length;i++) {
                string s=BSAs[i].Trim().ToLower();
                int index=s.IndexOf(';');
                if(index!=-1) s=s.Remove(index, s.Length-index);
                index=s.IndexOf('=');
                if(index!=-1) s=s.Remove(0, index+1);
                if(s.Trim()==FileName) return true;
            }
            return false;
        }
    }

    public static class MorrowindBSA {

        public static readonly List<string> Archives=new List<string>();
        private static bool UpdatedList=false;

        public static void ReadArchives() {
            Archives.AddRange(MorrowindINI.GetCleanINISection("[Archives]"));
        }

        public static void CommitArchives() {
            if(UpdatedList) {
                List<string> sec=new List<string>();
                for(int i=0;i<Archives.Count;i++) {
                    sec.Add("Archive "+i+"="+Archives[i]);
                }
                MorrowindINI.ReplaceINISection("Archives",  sec.ToArray());
            }
        }

        public static bool RegisterBSA(string name) {
            UpdatedList=true;
            string lname=name.ToLower();
            foreach(string s in Archives) {
                if(s.ToLower()==lname) return true;
            }
            Archives.Add(name);
            return false;
        }

        public static void UnregisterBSA(string name) {
            UpdatedList=true;
            name=name.ToLower();
            for(int i=0;i<Archives.Count;i++) {
                if(Archives[i].ToLower()==name) {
                    Archives.RemoveAt(i);
                    return;
                }
            }
        }

        public static string[] GetBSAEntries() {
            List<string> files=new List<string>();
            foreach(string path in Directory.GetFiles("data files", "*.bsa")) {
                BinaryReader br=null;
                br=new BinaryReader(File.OpenRead(path));
                if(Program.ReadCString(br)!="BSA") {
                    br.Close();
                    continue;
                }
                br.ReadUInt32();
                int offset=br.ReadInt32();
                br.ReadUInt32();
                int FolderCount=br.ReadInt32();
                int TotalFileCount=br.ReadInt32();
                int TotalFolderLength=br.ReadInt32();
                int TotalFileNameLength=br.ReadInt32();
                br.ReadUInt32();
                long FileNameBlockStart=offset+TotalFolderLength+16*TotalFileCount+17*FolderCount;
                long folderpos=br.BaseStream.Position;
                for(int i=0;i<FolderCount;i++) {
                    br.BaseStream.Position=folderpos+8;
                    int filecount=br.ReadInt32();
                    br.BaseStream.Position=br.ReadInt32()-TotalFileNameLength+1;
                    string fname=Program.ReadCString(br).Replace('/', '\\');
                    br.BaseStream.Position=FileNameBlockStart;
                    for(int j=0;j<filecount;j++) files.Add(fname+"\\"+Program.ReadCString(br));
                    FileNameBlockStart=br.BaseStream.Position;
                    folderpos+=16;
                }
                br.Close();
            }
            files.Sort();
            return files.ToArray();
        }
    }

}