using System;
using System.Collections.Generic;
using MessageBox=System.Windows.Forms.MessageBox;
using mbButtons=System.Windows.Forms.MessageBoxButtons;
using DialogResult=System.Windows.Forms.DialogResult;
using File=System.IO.File;

namespace MorrowindModManager {
    public static class ScriptRunner {
        private static ScriptReturnData srd;
        private static string DataFiles;
        private static string cLine="0";
        private static string[] SplitLine(string s) {
            List<string> temp=new List<string>();
            bool WasLastSpace=false;
            bool InQuotes=false;
            bool WasLastEscape=false;
            bool DoubleBreak=false;
            string CurrentWord="";

            for(int i=0;i<s.Length;i++) {
                switch(s[i]) {
                case ' ':
                    WasLastEscape=false;
                    if(InQuotes) {
                        CurrentWord+=s[i];
                    } else if(!WasLastSpace) {
                        temp.Add(CurrentWord);
                        CurrentWord="";
                        WasLastSpace=true;
                    }
                    break;
                case ';':
                    WasLastEscape=false;
                    if(!InQuotes) {
                        DoubleBreak=true;
                    } else CurrentWord+=s[i];
                    break;
                case '"':
                    if(InQuotes&&WasLastEscape) {
                        CurrentWord+=s[i];
                    } else if(InQuotes) {
                        InQuotes=false;
                    } else {
                        InQuotes=true;
                    }
                    WasLastSpace=false;
                    WasLastEscape=false;
                    break;
                case '\\':
                    if(InQuotes&&WasLastEscape) {
                        CurrentWord+=s[i];
                    } else if(InQuotes) {
                        WasLastEscape=true;
                    } else {
                        CurrentWord+=s[i];
                    }
                    WasLastSpace=false;
                    break;
                default:
                    WasLastEscape=false;
                    WasLastSpace=false;
                    CurrentWord+=s[i];
                    break;
                }
                if(DoubleBreak) break;
            }
            if(CurrentWord!="") temp.Add(CurrentWord);
            return temp.ToArray();
        }

        private static void Warn(string Message) {
            if(Settings.ShowScriptWarnings)
                MessageBox.Show(Message+" on line "+cLine, "Error in script");
        }

        private static bool FunctionIf(string[] line) {
            if(line.Length==1) {
                Warn("Missing arguments to function 'If'");
                return false;
            }
            switch(line[1]) {
            case "DialogYesNo":
                switch(line.Length) {
                case 2:
                    Warn("Missing arguments to function 'If DialogYesNo'");
                    return false;
                case 3:
                    return MessageBox.Show(line[2], "", mbButtons.YesNo)==DialogResult.Yes;
                case 4:
                    return MessageBox.Show(line[2], line[3], mbButtons.YesNo)==DialogResult.Yes;
                default:
                    Warn("Unexpected arguments after function 'If DialogYesNo'");
                    goto case 4;
                }
            case "DataFileExists":
                if(line.Length==2) {
                    Warn("Missing arguments to function 'If DataFileExists'");
                    return false;
                }
                return File.Exists("data files\\"+line[2]);
            case "VersionGreaterThan":
                if(line.Length==2) {
                    Warn("Missing arguments to function 'If VersionGreaterThan'");
                    return false;
                }
                try {
                    Version v=new Version(line[2]+".0");
                    Version v2=new Version(Program.MajorVersion.ToString()+"."+
                        Program.MinorVersion.ToString()+"."+Program.BuildNumber.ToString()+".0");
                    return (v2>v);
                } catch {
                    Warn("Invalid argument to function 'If VersionGreaterThan'");
                    return false;
                }
            case "VersionLessThan":
                if(line.Length==2) {
                    Warn("Missing arguments to function 'If VersionLessThan'");
                    return false;
                }
                try {
                    Version v=new Version(line[2]+".0");
                    Version v2=new Version(Program.MajorVersion.ToString()+"."+
                        Program.MinorVersion.ToString()+"."+Program.BuildNumber.ToString()+".0");
                    return (v2<v);
                } catch {
                    Warn("Invalid argument to function 'If VersionGreaterThan'");
                    return false;
                }
            default:
                Warn("Unknown argument '"+line[1]+"' supplied to 'If'");
                return false;
            }
            
        }

        private static string FunctionSelect(string[] line,List<string> CaseQueue,bool Previews) {
            if(line.Length<3) {
                Warn("Missing arguments to function 'Select'");
                return null;
            }
            Forms.SelectForm sf;
            if(!Previews) {
                string[] items=new string[line.Length-2];
                Array.Copy(line, 2, items, 0, items.Length);
                sf=new MorrowindModManager.Forms.SelectForm(items, line[1], CaseQueue!=null, null);
            } else {
                if(line.Length%2==1) Warn("Uneven number of arguments to 'SelectWithPreview'");
                string[] items=new string[( line.Length-(2-(line.Length%2)) ) / 2];
                string[] previews=new string[items.Length];
                for(int i=0;i<items.Length;i++) {
                    items[i]=line[i*2 + 2];
                    string file=line[i*2 + 3];
                    if(file=="None") {
                        previews[i]=null;
                    } else if(!Program.IsSafeFileName(line[i*2] + 2)) {
                        Warn("Preview file path was invalid");
                        previews[i]=null;
                    } else if(!File.Exists(DataFiles+file)) {
                        Warn("Preview file path des not exist");
                        previews[i]=null;
                    } else {
                        previews[i]=DataFiles+file;
                    }
                }
                sf=new MorrowindModManager.Forms.SelectForm(items, line[1], CaseQueue!=null, previews);
            }
            sf.ShowDialog();
            if(CaseQueue!=null) {
                for(int i=1;i<sf.SelectedIndex.Length;i++) {
                    CaseQueue.Insert(0,"Case "+line[sf.SelectedIndex[i]+2]);
                }
            }
            if(sf.SelectedIndex.Length==0) return "EndSelect";
            else return "Case "+line[sf.SelectedIndex[0]+2];
        }

        private static void FunctionMessage(string[] line) {
            switch(line.Length) {
            case 1:
                Warn("Missing arguments to function 'Message'");
                break;
            case 2:
                MessageBox.Show(line[1]);
                break;
            case 3:
                MessageBox.Show(line[1], line[2]);
                break;
            default:
                MessageBox.Show(line[1], line[2]);
                Warn("Unexpected arguments after 'Message'");
                break;
            }
        }

        private static void FunctionLoadOrder(string[] line, bool LoadAfter) {
            string WarnMess;
            if(LoadAfter) WarnMess = "function 'LoadAfter'"; else WarnMess="function 'LoadBefore'";
            if(line.Length<3) {
                Warn("Missing arguments to "+WarnMess);
                return;
            } else if(line.Length>3) {
                Warn("Unexpected arguments to "+WarnMess);
            }
            srd.LoadOrderList.Add(new PluginLoadInfo(line[1], line[2], LoadAfter));
        }

        private static void FunctionConflicts(string[] line,bool Conflicts) {
            string WarnMess;
            if(Conflicts) WarnMess="function 'ConflictsWith'"; else WarnMess="function 'DependsOn'";
            ConflictData cd=new ConflictData();
            cd.level=ConflictLevel.MajorConflict;
            switch(line.Length) {
            case 1:
                Warn("Missing arguments to "+WarnMess);
                return;
            case 2:
                cd.File=line[1];
                break;
            case 3:
                cd.Comment=line[2];
                goto case 2;
            case 4:
                switch(line[3]) {
                case "Unusable":
                    cd.level=ConflictLevel.Unusable;
                    break;
                case "Major":
                    cd.level=ConflictLevel.MajorConflict;
                    break;
                case "Minor":
                    cd.level=ConflictLevel.MinorConflict;
                    break;
                default:
                    Warn("Unknown conflict level after "+WarnMess);
                    break;
                }
                goto case 3;
            case 5:
                Warn("Unexpected arguments to "+WarnMess);
                goto case 4;
            case 6:
                cd.File=line[1];
                try {
                    cd.MinMajorVersion=Convert.ToInt32(line[2]);
                    cd.MinMinorVersion=Convert.ToInt32(line[3]);
                    cd.MaxMajorVersion=Convert.ToInt32(line[4]);
                    cd.MaxMinorVersion=Convert.ToInt32(line[5]);
                } catch {
                    Warn("Arguments to "+WarnMess+" in incorrect format");
                    return;
                }
                break;
            case 7:
                cd.Comment=line[6];
                goto case 6;
            case 8:
                switch(line[7]) {
                case "Unusable":
                    cd.level=ConflictLevel.Unusable;
                    break;
                case "Major":
                    cd.level=ConflictLevel.MajorConflict;
                    break;
                case "Minor":
                    cd.level=ConflictLevel.MinorConflict;
                    break;
                default:
                    Warn("Unknown conflict level after "+WarnMess);
                    break;
                }
                goto case 7;
            default:
                Warn("Unexpected arguments to "+WarnMess);
                goto case 8;
            }
            if(Conflicts) srd.ConflictsWith.Add(cd); else srd.DependsOn.Add(cd);
        }

        private static void FunctionModifyInstall(string[] line, bool Plugins, bool Install) {
            string WarnMess;
            if(Plugins) {
                if(Install) WarnMess="function 'InstallPlugin'"; else WarnMess="function 'DontInstallPlugin'";
            } else {
                if(Install) WarnMess="function 'InstallDataFile'"; else WarnMess="function 'DontInstallDataFile'";
            }
            if(line.Length==1) {
                Warn("Missing arguments to "+WarnMess);
                return;
            }
            if(line.Length>2) Warn("Unexpected arguments after "+WarnMess);
            //line[1]=line[1].ToLower();
            if(Plugins) {
                if(Install) {
                    srd.IgnorePlugins.Remove(line[1]);
                    if(!srd.InstallPlugins.Contains(line[1])) srd.InstallPlugins.Add(line[1]);
                } else {
                    srd.InstallPlugins.Remove(line[1]);
                    if(!srd.IgnorePlugins.Contains(line[1])) srd.IgnorePlugins.Add(line[1]);
                }
            } else {
                if(Install) {
                    srd.IgnoreData.Remove(line[1]);
                    if(!srd.InstallData.Contains(line[1])) srd.InstallData.Add(line[1]);
                } else {
                    srd.InstallData.Remove(line[1]);
                    if(!srd.IgnoreData.Contains(line[1])) srd.IgnoreData.Add(line[1]);
                }
            }
        }

        private static void FunctionRegisterBSA(string[] line,bool Register) {
            string WarnMess;
            if(Register) WarnMess="function 'RegisterBSA'"; else WarnMess="function 'UnregisterBSA'";
            if(line.Length==1) {
                Warn("Missing arguments to "+WarnMess);
                return;
            }
            if(line[1].Contains(",")||line[1].Contains(";")||line[1].Contains("=")) {
                Warn("BSA file names are not allowed to include the characters ',' '=' or ';'\n"+
                    "Invalid argument to "+WarnMess);
                return;
            }
            if(line.Length>2) Warn("Unexpected arguments after "+WarnMess);
            line[1]=line[1].ToLower();
            if(Register) {
                if(!srd.RegisterBSAList.Contains(line[1])) srd.RegisterBSAList.Add(line[1]);
            } else {
                srd.RegisterBSAList.Remove(line[1]);
            }
        }

        private static void FunctionUncheckESP(string[] line) {
            if(line.Length==1) {
                Warn("Missing arguments to UncheckESP");
                return;
            }
            if(line.Length>2) Warn("Unexpected arguments to UncheckESP");
            line[1]=line[1].ToLower();
            if(!srd.UncheckedPlugins.Contains(line[1])) srd.UncheckedPlugins.Add(line[1]);
        }

        private static void FunctionSetDeactivationWarning(string[] line) {
            if(line.Length<3) {
                Warn("Missing arguments to SetDeactivationWarning");
                return;
            }
            if(line.Length>3) Warn("Unexpected arguments to SetDeactivationWarning");
            line[1]=line[1].ToLower();
            for(int i=0;i<srd.EspDeactivation.Count;i++) {
                if(srd.EspDeactivation[i].Plugin==line[1]) srd.EspDeactivation.RemoveAt(i--);
            }
            switch(line[2]) {
            case "Allow":
                srd.EspDeactivation.Add(new ScriptEspWarnAgainst(line[1],DeactiveStatus.Allow));
                break;
            case "WarnAgainst":
                srd.EspDeactivation.Add(new ScriptEspWarnAgainst(line[1],DeactiveStatus.WarnAgainst));
                break;
            case "Disallow":
                srd.EspDeactivation.Add(new ScriptEspWarnAgainst(line[1],DeactiveStatus.Disallow));
                break;
            default:
                Warn("invalid argument to SetDeactivationWarning");
                return;
            }
        }

        public static void FunctionCopyDataFile(string[] line,bool Plugin) {
            if(line.Length<3) {
                Warn("Missing arguments to CopyDataFile");
                return;
            }
            if(line.Length>3) Warn("Unexpected arguments to CopyDataFile");
            line[1]=line[1].ToLower();
            line[2]=line[2].ToLower();
            if(!Program.IsSafeFileName(line[1])||!Program.IsSafeFileName(line[2])) {
                Warn("Invalid argument to CopyDataFile");
                return;
            }
            if(Plugin) {
                if(line[2].IndexOfAny(new char[] { System.IO.Path.DirectorySeparatorChar, System.IO.Path.AltDirectorySeparatorChar })!=-1) {
                    Warn("Plugins cannot be copied to subdirectories of the data folder");
                    return;
                }
                if(!(line[2].ToLower().EndsWith(".esp")||line[2].ToLower().EndsWith(".esm"))) {
                    Warn("Copied plugins must have a .esp or .esm extension");
                    return;
                }
            } else {
                if(line[2].ToLower().EndsWith(".esp")||line[2].ToLower().EndsWith(".esm")) {
                    Warn("Copied data files cannot have a .esp or .esm extension");
                    return;
                }
            }
            for(int i=0;i<srd.CopyDataFiles.Count;i++) {
                if(srd.CopyDataFiles[i].CopyTo==line[2]) srd.CopyDataFiles.RemoveAt(i--);
            }
            if(Plugin) srd.CopyPlugins.Add(new ScriptCopyDataFile(line[1], line[2]));
            else srd.CopyDataFiles.Add(new ScriptCopyDataFile(line[1],line[2]));
        }

        public static void FunctionEditINI(string[] line) {
            if(line.Length<4) {
                Warn("Missing arguments to EditINI");
                return;
            }
            if(line.Length>4) Warn("Unexpected arguments to EditINI");
            srd.INIEdits.Add(new INIEditInfo(line[1],line[2],line[3]));
        }

        public static void FunctionSetEspVar(string[] line, bool GMST) {
            string WarnMess;
            if(GMST) WarnMess="function 'SetGMST'"; else WarnMess="function 'SetGlobal'";
            if(line.Length<4) {
                Warn("Missing arguments to "+WarnMess);
                return;
            }
            if(line.Length>4) Warn("Unexpected extra arguments to "+WarnMess);
            if(!Program.IsSafeFileName(line[1])) {
                Warn("Illegal plugin name supplied to "+WarnMess);
                return;
            }
            srd.EspEdits.Add(new ScriptEspEdit(GMST, line[1].ToLower(), line[2].ToLower(), line[3]));
        }

        public static void FunctionDisplayFile(string[] line, bool Image) {
            string WarnMess;
            if(Image) WarnMess="function 'DisplayImage'"; else WarnMess="function 'DisplayText'";
            if(line.Length<2) {
                Warn("Missing arguments to "+WarnMess);
                return;
            }
            if(line.Length>3) Warn("Unexpected extra arguments to "+WarnMess);
            if(!Program.IsSafeFileName(line[1])) {
                Warn("Illegal path supplied to "+WarnMess);
                return;
            }
            if(!File.Exists(DataFiles+line[1])) {
                Warn("Non-existant file '"+line[1]+"' supplied to "+WarnMess);
                return;
            }
            if(Image) {
                System.Drawing.Image image=System.Drawing.Image.FromFile(DataFiles+line[1]);
                new ImageForm(image,(line.Length>2)?line[2]:line[1]).ShowDialog();
                image.Dispose();
            } else {
                string s=File.ReadAllText(DataFiles+line[1],System.Text.Encoding.Default);
                new TextEditor((line.Length>2)?line[2]:line[1], s, true, false).ShowDialog();
            }
        }

        public static ScriptReturnData ExecuteScript(string InputScript,string DataPath) {
            DataFiles=DataPath;
            srd=new ScriptReturnData();
            if(InputScript==null) return srd;
            string[] script=InputScript.Replace("\r", "").Split(new char[] { '\n' });
            string[] line;
            string s;
            int IfCount=0;
            string SkipTo=null;
            List<string> CaseQueue=new List<string>();
            for(int i=0;i<script.Length;i++) {
                cLine=i.ToString();
                s=script[i].Replace('\t', ' ').Replace(',', ' ').Trim();

                if(SkipTo!=null) {
                    if(s!=SkipTo) continue;
                    else SkipTo=null;
                }

                line=SplitLine(s);
                if(line.Length==0) continue;

                //If/Else/EndIf
                if(line[0]=="EndIf") {
                    if(line.Length>1) Warn("Unexpected arguments after 'EndIf'");
                    if(IfCount>0) IfCount--;
                    continue;
                } else if(line[0]=="Else") {
                    if(line.Length>1) Warn("Unexpected arguments after 'Else'");
                    if(IfCount==0) IfCount=1;
                    else if(IfCount==1) IfCount=0;
                    continue;
                } else if(line[0]=="If"&&IfCount>0) {
                    IfCount++;
                    continue;
                }
                if(IfCount>0) continue;
                
                switch(line[0]) {
                //Control structures
                case "Select":
                    SkipTo=FunctionSelect(line,null,false);
                    break;
                case "SelectMany":
                    SkipTo=FunctionSelect(line,CaseQueue,false);
                    break;
                case "SelectWithPreview":
                    SkipTo=FunctionSelect(line, null, true);
                    break;
                case "SelectManyWithPreview":
                    SkipTo=FunctionSelect(line, CaseQueue, true);
                    break;
                case "Break":
                    if(CaseQueue.Count>0) {
                        SkipTo=CaseQueue[0];
                        CaseQueue.RemoveAt(0);
                    } else SkipTo="EndSelect";
                    break;
                case "Case":
                    if(CaseQueue.Count>0&&CaseQueue[0]==s) CaseQueue.RemoveAt(0);
                    break;
                case "EndSelect":
                    break;
                case "If":
                    if(!FunctionIf(line)) IfCount=1;
                    break;
                case "IfNot":
                    if(FunctionIf(line)) IfCount=1;
                    break;
                    //Functions
                case "Message":
                    FunctionMessage(line);
                    break;
                case "LoadBefore":
                    FunctionLoadOrder(line, false);
                    break;
                case "LoadAfter":
                    FunctionLoadOrder(line,true);
                    break;
                case "ConflictsWith":
                    FunctionConflicts(line, true);
                    break;
                case "DependsOn":
                    FunctionConflicts(line, false);
                    break;
                case "DontInstallAnyPlugins":
                    srd.InstallAllPlugins=false;
                    break;
                case "DontInstallAnyDataFiles":
                    srd.InstallAllData=false;
                    break;
                case "InstallAllPlugins":
                    srd.InstallAllPlugins=true;
                    break;
                case "InstallAllDataFiles":
                    srd.InstallAllData=true;
                    break;
                case "InstallPlugin":
                    FunctionModifyInstall(line,true,true);
                    break;
                case "DontInstallPlugin":
                    FunctionModifyInstall(line,true,false);
                    break;
                case "InstallDataFile":
                    FunctionModifyInstall(line,false,true);
                    break;
                case "DontInstallDataFile":
                    FunctionModifyInstall(line,false,false);
                    break;
                case "RegisterBSA":
                    FunctionRegisterBSA(line,true);
                    break;
                case "UnregisterBSA":
                    FunctionRegisterBSA(line, false);
                    break;
                case "FatalError":
                    srd.CancelInstall=true;
                    break;
                case "UncheckESP":
                    FunctionUncheckESP(line);
                    break;
                case "SetDeactivationWarning":
                    FunctionSetDeactivationWarning(line);
                    break;
                case "CopyDataFile":
                    FunctionCopyDataFile(line,false);
                    break;
                case "CopyPlugin":
                    FunctionCopyDataFile(line, true);
                    break;
                case "EditINI":
                    FunctionEditINI(line);
                    break;
                case "SetGMST":
                    FunctionSetEspVar(line, true);
                    break;
                case "SetGlobal":
                    FunctionSetEspVar(line, false);
                    break;
                case "DisplayImage":
                    FunctionDisplayFile(line, true);
                    break;
                case "DisplayText":
                    FunctionDisplayFile(line, false);
                    break;
                default:
                    Warn("Unknown function '"+line[0]+"'");
                    break;
                }
                if(srd.CancelInstall) break;
            }
            if(SkipTo!=null) Warn("Expected "+SkipTo);
            //set srd to null so the garbage collector clears it up
            ScriptReturnData TempResult=srd;
            srd=null;
            return TempResult;
        }
    }
}
