using System;
using System.Windows.Forms;
using ICSharpCode.SharpZipLib.Zip;
using System.IO;
using System.Collections.Generic;

namespace MorrowindModManager {

    [Serializable]
    public class mmod {

        protected class PrivateData {
            public ZipFile modFile=null;
            public System.Drawing.Image image;
        }

        private static void WriteStreamToZip(BinaryWriter bw, Stream input) {
            input.Position=0;
            byte[] buffer=new byte[4096];
            int upto=0;
            while(input.Length-upto>4096) {
                input.Read(buffer, 0, 4096);
                bw.Write(buffer, 0, 4096);
                upto+=4096;
            }
            if(input.Length-upto>0) {
                input.Read(buffer, 0, (int)(input.Length-upto));
                bw.Write(buffer, 0, (int)(input.Length-upto));
            }
        }


        public static void Createmmod(mmodCreationOptions ops, string mmodFileName, ulong groups) {
            ZipOutputStream ZipStream;
            BinaryWriter mmodStream;
            ZipEntry ze;
            Stream DataCompressed;
            Stream DataInfo;
            mmodFileName=Settings.mmodDir+mmodFileName;

            ZipStream=new ZipOutputStream(File.Open(mmodFileName, FileMode.CreateNew));
            mmodStream=new BinaryWriter(ZipStream);
            try {
                ZipStream.SetLevel(ZipHandler.GetCompressionLevel(ops.mmodCompressionLevel));
                //The readme
                if(ops.readme!=""&&ops.readme!=null) {
                    ze=new ZipEntry("readme");
                    ZipStream.PutNextEntry(ze);
                    mmodStream.Write(ops.readme);
                    mmodStream.Flush();
                }
                //The script
                if(ops.script!=""&&ops.script!=null) {
                    ze=new ZipEntry("script");
                    ZipStream.PutNextEntry(ze);
                    mmodStream.Write(ops.script);
                    mmodStream.Flush();
                }
                //The image
                if(ops.Image!=""&&ops.Image!=null) {
                    ze=new ZipEntry("image");
                    ZipStream.PutNextEntry(ze);
                    FileStream fs=File.OpenRead(ops.Image);
                    WriteStreamToZip(mmodStream, fs);
                    mmodStream.Flush();
                    fs.Close();
                }
                //The config file
                ze=new ZipEntry("config");
                ZipStream.PutNextEntry(ze);
                mmodStream.Write(Program.CurrentmmodVersion);
                mmodStream.Write(ops.Name);
                mmodStream.Write(ops.MajorVersion);
                mmodStream.Write(ops.MinorVersion);
                mmodStream.Write(ops.Author);
                mmodStream.Write(ops.email);
                mmodStream.Write(ops.website);
                mmodStream.Write(ops.Description);
                mmodStream.Write(DateTime.Now.ToBinary());
                mmodStream.Write((byte)ops.CompressionType);
                mmodStream.Write(ops.BuildVersion);
                mmodStream.Flush();
                //plugins
                if(ops.esps.Length>0) {
                    ze=new ZipEntry("plugins.crc");
                    ZipStream.PutNextEntry(ze);
                    CompressionHandler.CompressFiles(ops.esps, ops.espPaths, out DataCompressed, out DataInfo,
                        ops.CompressionType, ops.DataFileCompresionLevel);
                    WriteStreamToZip(mmodStream, DataInfo);
                    mmodStream.Flush();
                    ZipStream.SetLevel(0);
                    ze=new ZipEntry("plugins");
                    ZipStream.PutNextEntry(ze);
                    WriteStreamToZip(mmodStream, DataCompressed);
                    mmodStream.Flush();
                    ZipStream.SetLevel(ZipHandler.GetCompressionLevel(ops.mmodCompressionLevel));
                    DataCompressed.Close();
                    DataInfo.Close();
                }
                //data files
                if(ops.DataFiles.Length>0) {
                    ze=new ZipEntry("data.crc");
                    ZipStream.PutNextEntry(ze);
                    CompressionHandler.CompressFiles(ops.DataFiles, ops.DataFilePaths, out DataCompressed, out DataInfo,
                        ops.CompressionType, ops.DataFileCompresionLevel);
                    WriteStreamToZip(mmodStream, DataInfo);
                    mmodStream.Flush();
                    ZipStream.SetLevel(0);
                    ze=new ZipEntry("data");
                    ZipStream.PutNextEntry(ze);
                    WriteStreamToZip(mmodStream, DataCompressed);
                    mmodStream.Flush();
                    ZipStream.SetLevel(ZipHandler.GetCompressionLevel(ops.mmodCompressionLevel));
                    DataCompressed.Close();
                    DataInfo.Close();
                }
                ZipStream.Finish();
            } finally {
                mmodStream.Close();
            }
            //Cleanup
            mmod o=Program.LoadNewmmod(mmodFileName);
            o.group=groups;
            Conflicts.UpdateConflict(o);
            Program.ClearTempFiles();
            GC.Collect();
        }

        public static void Remove(string filename) {
            filename=filename.ToLower();
            for(int i=0;i<Program.Data.mmods.Count;i++) {
                mmod o=Program.Data.mmods[i];
                if(o.LowerFileName==filename) {
                    if(o.Conflict==ConflictLevel.Active) {
                        o.DeletionDeactivate();
                    }
                    o.Close();
                    Program.Data.mmods.RemoveAt(i--);
                }
            }
        }

        [NonSerialized]
        private PrivateData pd=new PrivateData();
        public void RecreatePrivateData() { if(pd==null) pd=new PrivateData(); }

        public readonly string FilePath;
        public readonly string FileName;
        public readonly string LowerFileName;
        public readonly string ModName;
        public readonly int MajorVersion;
        public readonly int MinorVersion;
        public readonly int BuildVersion;
        public readonly string Description;
        public readonly string Email;
        public readonly string Website;
        public readonly string Author;
        public readonly DateTime CreationTime;
        public readonly string[] AllPlugins;
        public readonly DataFileInfo[] AllDataFiles;
        public readonly uint CRC;
        public readonly CompressionType CompType;
        private readonly byte FileVersion;

        public ulong group;
        public string Version {
            get { return ""+MajorVersion+((MinorVersion!=-1)?("."+MinorVersion+((BuildVersion!=-1)?("."+BuildVersion):"")):""); }
        }
        public string FullFilePath {
            get { if(FilePath==null) return Settings.mmodDir+FileName; else return FilePath+FileName; }
        }

        public string[] Plugins;
        public DataFileInfo[] DataFiles;
        public string[] BSAs;
        public List<INIEditInfo> INIEdits;

        public ConflictLevel Conflict=ConflictLevel.NoConflict;

        public readonly List<ConflictData> ConflictsWith=new List<ConflictData>();
        public readonly List<ConflictData> DependsOn=new List<ConflictData>();

        private ZipFile ModFile {
            get {
                if(pd.modFile!=null) return pd.modFile;
                pd.modFile=new ZipFile(FullFilePath);
                return pd.modFile;
            }
        }
        public System.Drawing.Image image {
            get {
                if(pd.image!=null) return pd.image;
                Stream s=ExtractWholeFile("image");
                if(s==null) return null;
                pd.image=System.Drawing.Image.FromStream(s);
                s.Close();
                return pd.image;
            }
        }

        public mmod(string path,bool InmmodDir) {
            BinaryReader br=null;
            try {
                if(InmmodDir) FilePath=null;
                else FilePath=Path.GetDirectoryName(path)+"\\";
                FileName=Path.GetFileName(path);
                LowerFileName=FileName.ToLower();
                Stream Config=ExtractWholeFile("config");
                if(Config==null) throw new mwmmException("Could not find mmod configuration data");
                br=new BinaryReader(Config);
                byte version=br.ReadByte();
                FileVersion=version;
                if(version>Program.CurrentmmodVersion) {
                    throw new mwmmException(FileName+" was created with a newer version of mwmm and could not be loaded");
                }
                ModName=br.ReadString();
                MajorVersion=br.ReadInt32();
                MinorVersion=br.ReadInt32();
                Author=br.ReadString();
                Email=br.ReadString();
                Website=br.ReadString();
                Description=br.ReadString();
                if(version>=2) {
                    CreationTime=DateTime.FromBinary(br.ReadInt64());
                } else {
                    string sCreationTime=br.ReadString();
                    if(!DateTime.TryParseExact(sCreationTime, "dd/MM/yyyy HH:mm", null, System.Globalization.DateTimeStyles.None, out CreationTime)) {
                        CreationTime=new DateTime(2006,1,1);
                    }
                }
                if(Description=="") Description="No description";
                CompType=(CompressionType)br.ReadByte();
                if(version>=1) {
                    BuildVersion=br.ReadInt32();
                } else BuildVersion=-1;

                AllPlugins=GetPluginList();
                AllDataFiles=GetDataList();
                foreach(string s2 in AllPlugins) {
                    if(!Program.IsSafeFileName(s2))
                        throw new mwmmException("File "+FileName+" appears to have been modified in a way which could cause a security risk."+
                            Environment.NewLine+"mwmm will not load it.");
                }
                foreach(string s2 in AllDataFiles) {
                    if(!Program.IsSafeFileName(s2))
                        throw new mwmmException("File "+FileName+" appears to have been modified in a way which could cause a security risk."+
                            Environment.NewLine+"mwmm will not load it.");
                }
                CRC=CompressionHandler.CRC(FullFilePath);
            } finally {
                if(br!=null) br.Close();
                Close();
            }
        }

        public void Close() {
            if(pd.modFile!=null) {
                pd.modFile.Close();
                pd.modFile=null;
            }
            if(pd.image!=null) {
                pd.image=null;
            }
        }

        private void CreateDirectoryStructure() {
            for(int x=0;x<DataFiles.Length;x++) {
                string s=Path.GetDirectoryName(DataFiles[x].FileName);
                if(!Directory.Exists("data files\\"+s)) Directory.CreateDirectory("data files\\"+s);
            }
        }

        private ScriptExecutationData ExecuteScript(string plugins, string data) {
            //Execute script
            ScriptReturnData srd=ScriptRunner.ExecuteScript(GetScript(),data);
            //return on fatal error
            if(srd.CancelInstall) return null;
            //Check that any required mods are already active
            foreach(ConflictData cd in srd.DependsOn) {
                if(!Program.Data.DoesModExist(cd, true)) {
                    string s="This mod depends on "+cd.File+", which is either not active or the wrong version";
                    if(cd.Comment!=null) s+="\n"+cd.Comment;
                    MessageBox.Show(s, "Error");
                    return null;
                }
            }
            foreach(ConflictData cd in srd.ConflictsWith) {
                if(Program.Data.DoesModExist(cd, true)) {
                    string s;
                    if(cd.Comment!=null) s="\n"+cd.Comment; else s="";
                    switch(cd.level) {
                    case ConflictLevel.Unusable:
                        MessageBox.Show("This mod has a fatal script defined conflict with "+cd.File+" and cant be activated."+s, "Error");
                        return null;
                    case ConflictLevel.MajorConflict:
                        s="This mod has a script defined major conflict with "+cd.File+s+"\nAre you sure you wish to activate it?";
                        break;
                    case ConflictLevel.MinorConflict:
                        s="This mod has a script defined minor conflict with "+cd.File+s+"\nAre you sure you wish to activate it?";
                        break;
                    }
                    if(MessageBox.Show(s, "Warning", MessageBoxButtons.YesNo)!=DialogResult.Yes) return null;
                }
            }
            //Update conflicts and dependencies
            ConflictsWith.Clear();
            ConflictsWith.AddRange(srd.ConflictsWith);
            DependsOn.Clear();
            DependsOn.AddRange(srd.DependsOn);
            //Create some variables
            List<string> strtemp1=new List<string>();
            //Fill the Plugins array with the plugins that need to be installed
            if(srd.InstallAllPlugins) strtemp1.AddRange(AllPlugins);
            foreach(string s in srd.InstallPlugins) { if(!Program.strArrayContains(strtemp1, s)) strtemp1.Add(s); }
            foreach(string s in srd.IgnorePlugins) { Program.strArrayRemove(strtemp1, s); }
            foreach(ScriptCopyDataFile scd in srd.CopyPlugins) {
                if(!File.Exists(plugins+scd.CopyFrom)) {
                    MessageBox.Show("The script attempted to copy the plugin '"+scd.CopyFrom+"', but it did not exist","Warning");
                } else {
                    if(File.Exists(plugins+scd.CopyTo)) File.Delete(plugins+scd.CopyTo);
                    File.Copy(plugins+scd.CopyFrom,plugins+scd.CopyTo);
                    if(!Program.strArrayContains(strtemp1, scd.CopyTo)) strtemp1.Add(scd.CopyTo);
                }
            }
            for(int i=0;i<strtemp1.Count;i++) if(!File.Exists(plugins+strtemp1[i])) strtemp1.RemoveAt(i--);
            Plugins=strtemp1.ToArray();
            strtemp1.Clear();
            //Fill the data file array with data files to be installed
            if(srd.InstallAllData) {
                for(int i=0;i<AllDataFiles.Length;i++) strtemp1.Add(AllDataFiles[i].FileName);
            }
            foreach(string s in srd.InstallData) { if(!Program.strArrayContains(strtemp1, s)) strtemp1.Add(s); }
            foreach(string s in srd.IgnoreData) { Program.strArrayRemove(strtemp1, s); }
            foreach(ScriptCopyDataFile scd in srd.CopyDataFiles) {
                if(!File.Exists(data+scd.CopyFrom)) {
                    MessageBox.Show("The script attempted to copy the data file '"+scd.CopyFrom+"', but it did not exist", "Warning");
                } else {
                    if(!Directory.Exists(Path.GetDirectoryName(data+scd.CopyTo))) Directory.CreateDirectory(Path.GetDirectoryName(data+scd.CopyTo));
                    if(File.Exists(data+scd.CopyTo)) File.Delete(data+scd.CopyTo);
                    File.Copy(data+scd.CopyFrom, data+scd.CopyTo);
                    if(!Program.strArrayContains(strtemp1, scd.CopyTo)) strtemp1.Add(scd.CopyTo);
                }
            }
            for(int i=0;i<strtemp1.Count;i++) if(!File.Exists(data+strtemp1[i])) strtemp1.RemoveAt(i--);
            List<DataFileInfo> dtemp1 = new List<DataFileInfo>();
            foreach(string s in strtemp1) {
                DataFileInfo dfi;//=Program.Data.GetDataFile(s);
                dfi=Program.strArrayGet(AllDataFiles, s);
                if(dfi!=null) dtemp1.Add(new DataFileInfo(dfi));
                else dtemp1.Add(new DataFileInfo(s, CompressionHandler.CRC(data+s)));
            }
            DataFiles=dtemp1.ToArray();
            strtemp1.Clear();
            dtemp1.Clear();
            //Register BSAs
            foreach(string s in srd.RegisterBSAList) {
                strtemp1.Add(s);
                BSA b=Program.Data.GetBSA(s);
                if(b==null) {
                    MorrowindBSA.RegisterBSA(s);
                    Program.Data.BSAs.Add(new BSA(s, FileName));
                } else {
                    b.UsedBy.Add(FileName);
                }
            }
            BSAs=strtemp1.ToArray();
            //Edit morrowind.ini files
            INIEdits=new List<INIEditInfo>();
            foreach(INIEditInfo iei in srd.INIEdits) {
                bool use=true;
                foreach(INIEditInfo edit in Program.Data.INIEdits) {
                    if(edit.Section==iei.Section&&edit.Name==iei.Name) {
                        MessageBox.Show("Unable to apply ini edit because '"+edit.Plugin+"' has already "+
                            "modified value "+edit.Section+edit.Name, "Error");
                        use=false;
                        break;
                    }
                }
                if(use) {
                    if(MessageBox.Show(FileName+" wants to edit morrowind.ini."+Environment.NewLine+
                        "Section: "+iei.Section+Environment.NewLine+
                        "Key: "+iei.Name+Environment.NewLine+
                        "New value: "+iei.NewValue+Environment.NewLine+
                        "Do you want to allow it?", "Warning", MessageBoxButtons.YesNo)!=DialogResult.Yes) continue;
                    iei.Plugin=FileName;
                    iei.OldValue=MorrowindINI.GetINIValue(iei.Section, iei.Name);
                    MorrowindINI.WriteINIValue(iei.Section, iei.Name, iei.NewValue);
                    Program.Data.INIEdits.Add(iei);
                    INIEdits.Add(iei);
                }
            }
            //return
            ScriptExecutationData sed=new ScriptExecutationData();
            sed.PluginOrder=srd.LoadOrderList.ToArray();
            sed.UncheckedPlugins=srd.UncheckedPlugins.ToArray();
            sed.EspDeactivationWarning=srd.EspDeactivation.ToArray();
            sed.EspEdits=srd.EspEdits.ToArray();
            return sed;
        }

        public void Activate(bool warn) {
            //Extract plugins and data files
            string PluginsPath=GetPlugins();
            string DataPath=GetDataFiles();
            //Run the attached script
            ScriptExecutationData sed=ExecuteScript(PluginsPath,DataPath);
            if(sed==null) return;
            //Final check for serious conflicts
            for(int i=0;i<Plugins.Length;i++) {
                if(Program.Data.DoesEspExist(Plugins[i])||
                    File.Exists("data files\\"+Plugins[i])) {
                    throw new mwmmException("Cannot extract plugin because a plugin with the same name already exists");
                }
            }
            //copy plugins
            for(int i=0;i<Plugins.Length;i++) {
                File.Move(PluginsPath+Plugins[i], "data files\\"+Plugins[i]);
                EspInfo ei=new EspInfo(Plugins[i], this);
                Program.Data.InsertESP(ei, sed.PluginOrder);
                foreach(string s in sed.UncheckedPlugins) {
                    if(s==Plugins[i].ToLower()) ei.Active=false;
                }
                foreach(ScriptEspWarnAgainst s in sed.EspDeactivationWarning) {
                    if(s.Plugin==Plugins[i].ToLower()) ei.Deactivatable=s.Status;
                }
                foreach(ScriptEspEdit see in sed.EspEdits) {
                    if(see.Plugin!=Plugins[i].ToLower()) continue;
                    try {
                        if(see.IsGMST) ConflictDetector.TesFile.SetGMST("data files\\"+see.Plugin, see.EDID, see.Value);
                        else ConflictDetector.TesFile.SetGLOB("data files\\"+see.Plugin, see.EDID, see.Value);
                    } catch(Exception ex) {
                        MessageBox.Show("An error occured editing plugin "+see.Plugin+"\n"+
                        ex.Message, "Error");
                    }
                }
            }
            //copy data files
            CreateDirectoryStructure();
            for(int i=0;i<DataFiles.Length;i++) {
                DataFileInfo dfi=null;
                if(Program.Data.DoesDataFileExist(DataFiles[i]) || File.Exists("data files\\"+DataFiles[i].FileName)) {
                    dfi=Program.Data.GetDataFile(DataFiles[i]);
                    uint CRC=DataFiles[i].CRC;
                    if(dfi!=null) {
                        dfi.AddOwner(this);
                        DataFiles[i]=dfi;
                    }
                    if(dfi==null||dfi.CRC!=CRC) {
                        if(warn&&MessageBox.Show(DataFiles[i].FileName+" already exists, but CRCs differ or are missing.\n"+
                            "Overwrite?", "Warning", MessageBoxButtons.YesNo)!=DialogResult.Yes) {
                            if(dfi==null) {
                                for(int j=i+1;j<DataFiles.Length;j++) DataFiles[j-1]=DataFiles[j];
                                Array.Resize<DataFileInfo>(ref DataFiles, DataFiles.Length-1);
                            }
                            continue;
                        } else if(dfi!=null) {
                            dfi.CRC=CRC;
                        }
                    }
                    File.Delete("data files\\"+DataFiles[i].FileName);
                } else {
                    Program.Data.DataFiles.Add(DataFiles[i]);
                }
                DataFiles[i].AddOwner(this);
                File.Move(DataPath+DataFiles[i].FileName, "data files\\"+DataFiles[i].FileName);
            }
            //mark the mod as active, update other mods conflicts and return
            Conflict=ConflictLevel.Active;
        }

        public bool Deactivate(bool Force) {
            if(Conflict!=ConflictLevel.Active) return true;
            //Check for any dependent mods that are still active
            foreach(mmod o in Program.Data.mmods) {
                if(o.Conflict!=ConflictLevel.Active||o==this) continue;
                foreach(ConflictData cd in DependsOn) {
                    if(cd==this) {
                        if(Force) {
                            o.Deactivate(true);
                        } else {
                            if(MessageBox.Show("Active mod "+o.FileName+" depends on "+FileName+" and must also be deactivated.\n"+
                            "Deactivate "+o.FileName+"?", "Question", MessageBoxButtons.YesNo)==DialogResult.Yes) {
                                if(!o.Deactivate(false)) return false;
                            } else return false;
                        }
                    }
                }
            }
            try {
                //Deactivate the mod
                Conflict=ConflictLevel.NoConflict;
                ConflictsWith.Clear();
                DependsOn.Clear();
                //Undo any ini or xml edits
                foreach(INIEditInfo iei in INIEdits) {
                    MorrowindINI.WriteINIValue(iei.Section, iei.Name, iei.OldValue);
                    Program.Data.INIEdits.Remove(iei);
                }
                INIEdits=null;
                //Clear out the plugins
                foreach(string s in Plugins) {
                    EspInfo ei=Program.Data.GetEsp(s);
                    if(ei!=null) Program.Data.Esps.Remove(ei);
                    File.Delete("data files\\"+s);
                }
                //Clear out the data files
                foreach(DataFileInfo dfi in DataFiles) dfi.RemoveOwner(this);
                DataFiles=new DataFileInfo[0];
                //Unregister BSAs
                for(int i=0;i<Program.Data.BSAs.Count;i++) {
                    BSA b=(BSA)Program.Data.BSAs[i];
                    if(b.UsedBy.Contains(FileName)) b.UsedBy.Remove(FileName);
                    if(b.UsedBy.Count==0) {
                        MorrowindBSA.UnregisterBSA(b.FileName);
                        Program.Data.BSAs.RemoveAt(i--);
                    }
                }
            } catch(Exception ex) {
                if(Force) return true;
                MessageBox.Show("An error occured trying to deactivate "+FileName+".\n"+ex.Message, "Error");
                return false;
            }
            return true;
        }

        public void DeletionDeactivate() {
            //Undo any ini or xml edits
            if(INIEdits!=null) foreach(INIEditInfo iei in INIEdits) {
                    Program.Data.INIEdits.Remove(iei);
                }
            INIEdits=null;
            //Clear out the plugins
            foreach(string s in Plugins) {
                EspInfo ei=Program.Data.GetEsp(s);
                if(ei!=null) Program.Data.Esps.Remove(ei);
                File.Delete("data files\\"+s);
            }
            //Clear out the data files
            foreach(DataFileInfo dfi in DataFiles) dfi.RemoveOwner(this);
            DataFiles=new DataFileInfo[0];
        }

        public void Clean() {
            int a=0, b=0, c=0;
            Clean(ref a, ref b, ref c);
        }
        public void Clean(ref int DeletedCount, ref int SkippedCount, ref int NotFoundCount) {
            //scan for ghost data file links
            //Not required anymore?
            //for(int i=0;i<Program.Data.DataFiles.Count;i++) {
            //    DataFileInfo dfi=Program.Data.DataFiles[i];
            //    if(dfi.UsedBy.Contains(LowerFileName)) dfi.UsedBy.Remove(LowerFileName);
            //    if(dfi.UsedBy.Count==0) Program.Data.DataFiles.RemoveAt(i--);
            //}
            //delete plugins
            foreach(string s in AllPlugins) {
                if(!File.Exists("data files\\"+s)) {
                    NotFoundCount++;
                    continue;
                }
                EspInfo ei=Program.Data.GetEsp(s);
                if(ei==null||ei.BelongsTo==EspInfo.UnknownOwner) {
                    File.Delete("data files\\"+s);
                    if(ei!=null) Program.Data.Esps.Remove(ei);
                    DeletedCount++;
                } else SkippedCount++;
            }
            //delete data files
            for(int i=0;i<AllDataFiles.Length;i++) {
                string s=AllDataFiles[i].FileName;
                if(!File.Exists("data files\\"+s)) {
                    NotFoundCount++;
                    continue;
                }
                if(!Program.Data.DoesDataFileExist(s)) {
                    File.Delete("data files\\"+s);
                    DeletedCount++;
                } else SkippedCount++;
            }
        }

        private string[] GetPluginList() {
            Stream TempStream=ExtractWholeFile("plugins.crc");
            if(TempStream==null) return new string[0];
            BinaryReader br=new BinaryReader(TempStream);
            List<string> ar=new List<string>();
            while(br.PeekChar()!=-1) {
                ar.Add(br.ReadString());
                br.ReadInt32();
                br.ReadInt64();
            }
            br.Close();
            return ar.ToArray();
        }

        private DataFileInfo[] GetDataList() {
            Stream TempStream=ExtractWholeFile("data.crc");
            if(TempStream==null) return new DataFileInfo[0];
            BinaryReader br=new BinaryReader(TempStream);
            List<DataFileInfo> ar=new List<DataFileInfo>();
            while(br.PeekChar()!=-1) {
                string s=br.ReadString();
                ar.Add(new DataFileInfo(s, br.ReadUInt32()));
                br.ReadInt64();
            }
            br.Close();
            return ar.ToArray();
        }

        public string GetPlugins() {
            return ParseCompressedStream("plugins.crc", "plugins");
        }

        public string GetDataFiles() {
            return ParseCompressedStream("data.crc", "data");
        }

        public string GetReadme() {
            Stream s=ExtractWholeFile("readme");
            if(s==null) return null;
            BinaryReader br=new BinaryReader(s);
            string readme=br.ReadString();
            br.Close();
            return readme;
        }

        public string GetScript() {
            Stream s=ExtractWholeFile("script");
            if(s==null) return null;
            BinaryReader br=new BinaryReader(s);
            string script=br.ReadString();
            br.Close();
            return script;
        }

        public string GetImage() {
            string BitmapPath="";
            Stream s=ExtractWholeFile("image", ref BitmapPath);
            if(s==null) BitmapPath=null;
            else s.Close();
            return BitmapPath;
        }

        public string GetInfo() {
            string n=Environment.NewLine;
            string report=FileName+n+n+"[basic info]"+n+"Name: "+ModName+n;
            report+=n+"Author: "+Author+n+"version: "+Version+
            	n+"Contact: "+Email+n+"Website: "+Website+n+n+Description+n;
            report+=n+"Date this mmod was compiled: "+CreationTime.ToString();
            report+=n+"Contains readme: ";
            if(DoesReadmeExist()) report+="yes"; else report+="no";
            report+=n+"Contains script: ";
            if(DoesScriptExist()) report+="yes"; else report+="no";
            report+=n+n+"[mmod file information]"+n;
            FileInfo fi;
            fi=new FileInfo(FullFilePath);
            if(fi.Length<8192) { //8kb
                report+="File size: "+fi.Length.ToString()+" bytes"+n;
            } else if(fi.Length<8388608) { //8mb
                report+="File size: "+(fi.Length/1024).ToString()+" kilobytes"+n;
            } else {
                report+="File size: "+(fi.Length/1048576).ToString()+" megabytes"+n;
            }
            report+="internal mmod file version: "+FileVersion+n;
            report+="CRC: "+CRC.ToString("x").ToUpper().PadLeft(8, '0')+n;
            report+="Created or installed: "+fi.CreationTime.ToShortDateString()+" "+fi.CreationTime.ToShortTimeString()+n;
            report+="Last modified: "+fi.LastWriteTime.ToShortDateString()+" "+fi.LastWriteTime.ToShortTimeString()+n;
            if(AllPlugins.Length>0) {
                report+=n+"[Complete plugin list]"+n;
                foreach(string s in AllPlugins) report+=s+n;
            }
            if(AllDataFiles.Length>0) {
                report+=n+"[Complete data file list]"+n;
                foreach(DataFileInfo df in AllDataFiles)
                    report+=df.FileName.PadRight(80)+" ("+df.CRC.ToString("x").ToUpper().PadLeft(8, '0')+")"+n;
            }
            if(Conflict==ConflictLevel.Active) {
                if(Plugins.Length>0) {
                    report+=n+"[Currently installed plugin list]"+n;
                    foreach(string s in Plugins) report+=s+n;
                }
                if(DataFiles.Length>0) {
                    report+=n+"[Currently installed data files]"+n;
                    foreach(DataFileInfo dfi in DataFiles) report+=dfi.FileName+n;
                }
                if(BSAs.Length>0) {
                    report+=n+"[Registered BSAs]"+n;
                    foreach(string s in BSAs) report+=s+n;
                }
            }
            return report;
        }

        public bool DoesReadmeExist() {
            ZipEntry ze=ModFile.GetEntry("readme");
            return (ze!=null);
        }

        public bool DoesScriptExist() {
            ZipEntry ze=ModFile.GetEntry("script");
            return (ze!=null);
        }

        private string ParseCompressedStream(string fileList, string compressedStream) {
            string path;
            Stream FileList=ExtractWholeFile(fileList);
            if(FileList==null) return null;
            Stream CompressedStream=ExtractWholeFile(compressedStream);
            path=CompressionHandler.DecompressFiles(FileList, CompressedStream, CompType);
            FileList.Close();
            CompressedStream.Close();
            return path;
        }

        public string GetConfig() {
            string s="";
            Stream st=ExtractWholeFile("config", ref s);
            st.Close();
            return s;
        }

        private Stream ExtractWholeFile(string s) {
            string s2=null;
            return ExtractWholeFile(s, ref s2);
        }
        private Stream ExtractWholeFile(string s, ref string path) {
            ZipEntry ze=ModFile.GetEntry(s);
            if(ze==null) return null;
            return ExtractWholeFile(ze, ref path);
        }
        private Stream ExtractWholeFile(ZipEntry ze, ref string path) {
            Stream file=ModFile.GetInputStream(ze);
            Stream TempStream;
            if(path!=null||ze.Size>Settings.MaxMemoryStreamSize) {
                TempStream=Program.CreateTempFile(out path);
            } else {
                TempStream=new MemoryStream((int)ze.Size);
            }
            byte[] buffer=new byte[4096];
            int i;
            while((i=file.Read(buffer, 0, 4096))>0) {
                TempStream.Write(buffer, 0, i);
            }
            TempStream.Position=0;
            return TempStream;
        }

        private void ReplaceFileInmmod(string file, string contents) {
            Close();
            FastZip fz=new FastZip();
            string dir=Program.CreateTempDirectory();
            fz.ExtractZip(Settings.mmodDir+FileName, dir, null);
            if(contents==null||contents=="") {
                File.Delete(dir+file);
            } else {
                BinaryWriter bw=new BinaryWriter(File.Open(dir+file, FileMode.Create));
                bw.Write(contents);
                bw.Close();
            }
            string dir2=Program.CreateTempDirectory();
            fz.CreateZip(dir2+FileName, dir, false, null);
            File.Delete(Settings.mmodDir+FileName);
            File.Move(dir2+FileName, Settings.mmodDir+FileName);
        }

        public void ReplaceReadme(string readme) {
            ReplaceFileInmmod("readme", readme);
        }

        public void ReplaceScript(string script) {
            ReplaceFileInmmod("script", script);
        }

    }
}