using System;
using System.Collections.Generic;
using System.IO;

namespace ConflictDetector {
    [Serializable]
    public class HeaderInfo {
        public string Author;
        public string Description;
    }

    public static class TesFile {
        private static string ReadRecName(BinaryReader br) {
            byte[] b = new byte[4];
            br.Read(b, 0, 4);
            return ""+((char)b[0])+((char)b[1])+((char)b[2])+((char)b[3]);
        }
        private static string ReadCString(BinaryReader br) {
            string s="";
            byte b;
            while(true) {
                b=br.ReadByte();
                if(b==0) break;
                s+=(char)b;
            }
            return s;
        }

        public static void SetGMST(string file, string edid, string value) {
            switch(edid.ToLower()[0]) {
            case 'f':
                float fparse;
                if(!float.TryParse(value, out fparse))
                    throw new MorrowindModManager.mwmmException("Invalid value for float gmst");
                break;
            case 'i':
                int iparse;
                if(!int.TryParse(value, out iparse))
                    throw new MorrowindModManager.mwmmException("Invalid value of integer gmst");
                break;
            case 's':
                break;
            default:
                throw new MorrowindModManager.mwmmException("Unknown gmst type");
            }
            File.Delete(MorrowindModManager.Program.TempDir+"tempesp");
            File.Move(file, MorrowindModManager.Program.TempDir+"tempesp");
            BinaryReader br=new BinaryReader(File.OpenRead(MorrowindModManager.Program.TempDir+"tempesp"));
            BinaryWriter bw=null;
            try {
                string s;
                int size;
                long loc1=0;
                long loc2=0;
                int gsize=0;
                s=ReadRecName(br);
                if(s!="TES4") {
                    br.Close();
                    File.Move(MorrowindModManager.Program.TempDir+"tempesp",file);
                    throw new MorrowindModManager.mwmmException("file was not a valid tes4 plugin");
                }
                bw=new BinaryWriter(File.Create(file));
                size=br.ReadInt32();
                br.BaseStream.Position=0;
                bw.Write(br.ReadBytes(size+20));
                while(br.PeekChar()!=-1) {
                    s=ReadRecName(br);
                    if(s=="GRUP") {
                        loc1=0;
                        size=br.ReadInt32();
                        gsize=size;
                        s=ReadRecName(br);
                        br.BaseStream.Position-=12;
                        if(s=="GMST") {
                            loc1=br.BaseStream.Position+4;
                            bw.Write(br.ReadBytes(20));
                        } else {
                            bw.Write(br.ReadBytes(size));
                        }
                        continue;
                    }
                    if(s=="GMST") {
                        loc2=br.BaseStream.Position;
                        size=br.ReadInt32();
                        br.BaseStream.Position-=8;
                        bw.Write(br.ReadBytes(20));
                        s=ReadRecName(br);
                        if(s!="EDID") {
                            br.BaseStream.Position-=4;
                            bw.Write(br.ReadBytes(size));
                            continue;
                        }
                        br.BaseStream.Position+=2;
                        s=ReadCString(br);
                        if(s.ToLower()!=edid) {
                            br.BaseStream.Position=loc2+16;
                            bw.Write(br.ReadBytes(size));
                            continue;
                        }
                        long loc3;
                        switch(s.ToLower()[0]) {
                        case 'f':
                            loc3=br.BaseStream.Position+6;
                            br.BaseStream.Position=loc2+16;
                            bw.Write(br.ReadBytes(size));
                            bw.Seek((int)loc3, SeekOrigin.Begin);
                            bw.Write(float.Parse(value));
                            bw.Seek(0, SeekOrigin.End);
                            break;
                        case 'i':
                            loc3=br.BaseStream.Position+6;
                            br.BaseStream.Position=loc2+16;
                            bw.Write(br.ReadBytes(size));
                            bw.Seek((int)loc3, SeekOrigin.Begin);
                            bw.Write(int.Parse(value));
                            bw.Seek(0, SeekOrigin.End);
                            break;
                        case 's':
                            loc3=br.BaseStream.Position+6;
                            long savedpos=loc2+16+size;
                            br.BaseStream.Position=loc2+16;
                            bw.Write(br.ReadBytes((int)(loc3-(loc2+18))));
                            bw.Write((short)(value.Length+1));
                            foreach(char c in value) bw.Write((byte)c);
                            bw.Write((byte)0);
                            bw.Seek((int)loc2, SeekOrigin.Begin);
                            int size2=value.Length+edid.Length+14;
                            bw.Write(size2);
                            if(loc1!=0) {
                                bw.Seek((int)loc1, SeekOrigin.Begin);
                                bw.Write(gsize-(size-size2));
                            }
                            bw.Seek(0, SeekOrigin.End);
                            br.BaseStream.Position=savedpos;
                            break;
                        }
                        bw.Write(br.ReadBytes((int)(br.BaseStream.Length-br.BaseStream.Position)));
                    } else {
                        size=br.ReadInt32();
                        br.BaseStream.Position-=8;
                        bw.Write(br.ReadBytes(size+20));
                    }
                }
            } finally {
                if(br!=null) br.Close();
                if(bw!=null) bw.Close();
            }
        }
        public static void SetGLOB(string file, string edid, string value) {
            long offset=-1;
            char type='\0';
            float f=0;
            int l=0;
            short s=0;
            string str;
            int size;

            BinaryReader br=new BinaryReader(File.OpenRead(file));
            try {
                str=ReadRecName(br);
                if(str!="TES4")
                    throw new MorrowindModManager.mwmmException("file was not a valid tes4 plugin");
                size=br.ReadInt32();
                br.BaseStream.Position+=size+12;

                while(br.PeekChar()!=-1) {
                    str=ReadRecName(br);
                    if(str=="GRUP") {
                        size=br.ReadInt32();
                        str=ReadRecName(br);
                        if(str!="GLOB") {
                            br.BaseStream.Position+=size-12;
                        } else {
                            br.BaseStream.Position+=8;
                        }
                        continue;
                    }
                    if(str=="GLOB") {
                        size=br.ReadInt32();
                        br.BaseStream.Position+=12;
                        long nextrecord=br.BaseStream.Position+size;
                        if(ReadRecName(br)!="EDID") {
                            br.BaseStream.Position=nextrecord;
                            continue;
                        }
                        br.ReadInt16();
                        str=ReadCString(br);
                        if(str!=edid) {
                            br.BaseStream.Position=nextrecord;
                            continue;
                        }
                        if(ReadRecName(br)!="FNAM") {
                            br.BaseStream.Position=nextrecord;
                            continue;
                        }
                        br.ReadInt16();
                        type=(char)br.ReadByte();
                        switch(type) {
                        case 'f':
                            if(!float.TryParse(value, out f))
                                throw new MorrowindModManager.mwmmException("New global value was not in the required format");
                            break;
                        case 'l':
                            if(!int.TryParse(value, out l))
                                throw new MorrowindModManager.mwmmException("New global value was not in the required format");
                            break;
                        case 's':
                            if(!short.TryParse(value, out s))
                                throw new MorrowindModManager.mwmmException("New global value was not in the required format");
                            break;
                        }
                        if(ReadRecName(br)!="FLTV") {
                            br.BaseStream.Position=nextrecord;
                            continue;
                        }
                        br.ReadInt16();
                        offset=br.BaseStream.Position;
                        break;
                    }
                }
            } finally {
                br.Close();
            }

            if(offset!=-1) {
                BinaryWriter bw=new BinaryWriter(File.Open(file, FileMode.Open, FileAccess.Write));
                bw.Seek((int)offset, SeekOrigin.Begin);
                switch(type) {
                case 'f':
                    bw.Write(f);
                    break;
                case 'l':
                    bw.Write((float)l);
                    break;
                case 's':
                    bw.Write((float)s);
                    break;
                }
                bw.Close();
            } else {
                throw new MorrowindModManager.mwmmException("The specified global EDID was not found");
            }
        }
        public static HeaderInfo GetHeader(string FilePath) {
            BinaryReader br=new BinaryReader(File.OpenRead(FilePath));
            HeaderInfo hi=new HeaderInfo();
            try {
                string s;
                uint size;
                s=ReadRecName(br);
                if(s!="TES4") return hi;
                size=br.ReadUInt32();
                br.BaseStream.Position+=12;
                uint read=0;
                while(read<size) {
                    s=ReadRecName(br);
                    ushort subsize=br.ReadUInt16();
                    if(s=="CNAM") hi.Author=ReadCString(br);
                    else if(s=="SNAM") hi.Description=ReadCString(br);
                    else br.BaseStream.Position+=subsize;
                    read+=(uint)(subsize+6);
                }
            } finally {
                br.Close();
            }
            return hi;
        }
        public static EDID[] GetIDList(string FilePath, plugin p) {
            List<EDID> ids=new List<EDID>();
            BinaryReader br=new BinaryReader(File.OpenRead(FilePath));
            try {
                string s;
                uint size;
                s=ReadRecName(br);
                if(s!="TES4") return new EDID[0];
                size=br.ReadUInt32();
                br.BaseStream.Position+=size+12;
                while(br.PeekChar()!=-1) {
                    s=ReadRecName(br);
                    size=br.ReadUInt32();
                    uint comp=br.ReadUInt32();
                    br.BaseStream.Position+=8;
                    if(s=="GRUP") continue;
                    if((comp&0x00040000)>0) {
                        ICSharpCode.SharpZipLib.Zip.Compression.Inflater inf;
                        inf=new ICSharpCode.SharpZipLib.Zip.Compression.Inflater();
                        byte[] In=new byte[size-4];
                        byte[] Out=new byte[br.ReadUInt32()];
                        br.Read(In, 0, (int)(size-4));
                        inf.SetInput(In);
                        inf.Inflate(Out);
                        if((byte)Out[0]=='E'&&(byte)Out[1]=='D'&&(byte)Out[2]=='I'&&(byte)Out[3]=='D') {
                            string s2="";
                            int i=6;
                            while(Out[i]!=0) {
                                s2+=(char)Out[i++];
                            }
                            ids.Add(new EDID(s2, s, p));
                        }
                    } else {
                        string s2=ReadRecName(br);
                        if(s2=="EDID") {
                            s2="";
                            byte b;
                            ushort size2=br.ReadUInt16();
                            while((b=br.ReadByte())!=0) {
                                s2+=(char)b;
                            }
                            br.BaseStream.Position+=size-(6+size2);
                            ids.Add(new EDID(s2, s, p));
                        } else {
                            br.BaseStream.Position+=size-4;
                        }
                    }

                }
            } finally {
                br.Close();
            }
            return ids.ToArray();
        }
        private static string[] GetDataFileList(byte[] record) {
            List<string> ids=new List<string>();
            MemoryStream ms=new MemoryStream(record);
            BinaryReader br=new BinaryReader(ms);
            int size=record.Length;
            int read=0;
            while(read<size) {
                string s=ReadRecName(br);
                ushort size2=br.ReadUInt16();
                if(s=="MODL"||s=="ICON") {
                    string s2="";
                    byte b;
                    while((b=br.ReadByte())!=0) {
                        s2+=(char)b;
                    }
                    switch(s) {
                    case "MODL": ids.Add("meshes\\"+s2.ToLower()); break;
                    case "ICON": ids.Add("textures\\"+s2.ToLower()); break;
                    }
                } else {
                    br.BaseStream.Position+=size2;
                }
                read+=size2+6;
            }
            ms.Close();
            return ids.ToArray();
        }
        public static string[] GetDataFileList(string FilePath) {
            List<string> ids=new List<string>();
            BinaryReader br=new BinaryReader(File.OpenRead(FilePath));
            try {
                string s;
                uint size;
                s=ReadRecName(br);
                if(s!="TES4") return new string[0];
                size=br.ReadUInt32();
                br.BaseStream.Position+=size+12;
                while(br.PeekChar()!=-1) {
                    s=ReadRecName(br);
                    size=br.ReadUInt32();

                    uint comp=br.ReadUInt32();
                    br.BaseStream.Position+=8;
                    if(s=="GRUP") continue;
                    if((comp&0x00040000)>0) {
                        ICSharpCode.SharpZipLib.Zip.Compression.Inflater inf;
                        inf=new ICSharpCode.SharpZipLib.Zip.Compression.Inflater();
                        byte[] In=new byte[size-4];
                        byte[] Out=new byte[br.ReadUInt32()];
                        br.Read(In, 0, (int)(size-4));
                        inf.SetInput(In);
                        inf.Inflate(Out);
                        try {
                            ids.AddRange(GetDataFileList(Out));
                        } catch { }
                    } else {
                        try {
                            ids.AddRange(GetDataFileList(br.ReadBytes((int)size)));
                        } catch { }
                    }
                }
            } finally {
                br.Close();
            }
            return ids.ToArray();
        }
    }
}