using System;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;
using System.IO;

namespace MorrowindModManager {
    public partial class MainForm : Form {
        public MainForm() {
            InitializeComponent();
            lvEspList.ListViewItemSorter=new EspListSorter();
            lvModList.ListViewItemSorter=new mmodListSorter();
            UpdateEspList();
            UpdatemmodList();
            UpdateGroups();
            if(Settings.mfSize!=System.Drawing.Size.Empty) Size=Settings.mfSize;
            if(Settings.mfMaximized) WindowState=System.Windows.Forms.FormWindowState.Maximized;
            if(Settings.mfSplitterPos!=0) splitContainer1.SplitterDistance=Settings.mfSplitterPos;
            if(Settings.EspColWidth1!=-1) lvEspList.Columns[0].Width=Settings.EspColWidth1;
            if(Settings.EspColWidth2!=-1) lvEspList.Columns[1].Width=Settings.EspColWidth2;
            cmbEspSortOrder.SelectedIndex=Settings.EspSortOrder;
            cmbmmodSortOrder.SelectedIndex=Settings.mmodSortOrder;
        }

        private void RefreshLists() {
            if(Settings.AutoUpdateConflicts) Conflicts.UpdateConflicts();
            UpdateEspList();
            UpdatemmodList();
        }

        private void UpdateEspList() {
            lvEspList.SuspendLayout();
            lvEspList.Items.Clear();
            int ActiveCount=0;
            foreach(EspInfo ei in Program.Data.Esps) {
                string toolText=ei.FileName+"\nAuthor: "+ei.header.Author;
                ListViewItem lvi=new ListViewItem(new string[] { ei.FileName, ei.BelongsTo });
                if(ei.Active) {
                    lvi.Checked=true;
                    toolText+="\n\nFormID: "+(ActiveCount++).ToString("x").PadLeft(2, '0');
                }
                if(ei.header.Description!=null) {
                    toolText+="\n\n"+ei.header.Description;
                }
                lvi.Tag=ei;
                lvi.ToolTipText=toolText;
                lvEspList.Items.Add(lvi);
            }
            lvEspList.ResumeLayout();
        }

        private void UpdatemmodList() {
            lvModList.OwnerDraw=true;
            lvModList.SuspendLayout();
            int pos;
            if(lvModList.SelectedIndices.Count>0) pos=lvModList.SelectedIndices[0];
            else pos=0;
            lvModList.Clear();
            foreach(mmod o in Program.Data.mmods) {
                if(cmbGroupFilter.SelectedIndex!=0) {
                    if(cmbGroupFilter.SelectedIndex==1) {
                        if(o.group!=0) continue;
                    } else {
                        if((o.group&(ulong)(0x01<<(cmbGroupFilter.SelectedIndex-2))) == 0) continue;
                    }
                }
                ListViewItem lvi=new ListViewItem(Path.GetFileNameWithoutExtension(o.FileName));
                lvi.ToolTipText=o.ModName+"\nAuthor: "+o.Author+"\n"+"version: "+o.Version+"\n\n"+o.Description;
                lvi.Tag=o;
                switch(o.Conflict) {
                case ConflictLevel.Active:
                    lvi.ImageIndex=0;
                    break;
                case ConflictLevel.NoConflict:
                    lvi.ImageIndex=1;
                    break;
                case ConflictLevel.MinorConflict:
                    lvi.ImageIndex=2;
                    break;
                case ConflictLevel.MajorConflict:
                    lvi.ImageIndex=3;
                    break;
                case ConflictLevel.Unusable:
                    lvi.ImageIndex=4;
                    break;
                default:
                    MessageBox.Show("mmod had unrecognised conflict level", "Error");
                    lvi.ImageIndex=1;
                    break;
                }
                lvModList.Items.Add(lvi);
            }
            if(lvModList.Items.Count>pos) lvModList.EnsureVisible(pos);
            lvModList.ResumeLayout();
            lvModList.OwnerDraw=false;
        }

        private void lvEspList_ItemCheck(object sender, ItemCheckEventArgs e) {
            if(((EspInfo)(lvEspList.Items[e.Index].Tag)).Active==(e.NewValue==CheckState.Checked)) return;
            if(e.NewValue==CheckState.Checked) {
                ((EspInfo)(lvEspList.Items[e.Index].Tag)).Active=true;
                return;
            }
            EspInfo ei=(EspInfo)lvEspList.Items[e.Index].Tag;
            if(ei.Parent!=null) {
                if(ei.Deactivatable==DeactiveStatus.Disallow) {
                    MessageBox.Show("That plugin belongs to "+ei.BelongsTo+".\n"+
                    "You must disable the entire mod instead of just this single file.", "Error");
                    e.NewValue=e.CurrentValue;
                    return;
                } else if(ei.Deactivatable==DeactiveStatus.WarnAgainst) {
                    if(MessageBox.Show("That plugin belongs to "+ei.BelongsTo+".\n"+
                    "It is recommended that you disable the entire mod instead of just this single file.\n"+
                    "Are you sure you wish to continue?", "Warning", MessageBoxButtons.YesNo)!=DialogResult.Yes) {
                        e.NewValue=e.CurrentValue;
                        return;
                    }
                }
            }
            ((EspInfo)(lvEspList.Items[e.Index].Tag)).Active=false;
        }

        private void lvModList_ItemActivate(object sender, EventArgs e) {
            if(lvModList.SelectedItems.Count!=1) return;
            Activatemmod((mmod)lvModList.SelectedItems[0].Tag, false);
            RefreshLists();
        }

        private void bCreate_Click(object sender, EventArgs e) {
            if(CreateModForm.ShowForm()) {
                UpdatemmodList();
            }
        }

        private void deleteToolStripMenuItem_Click(object sender, EventArgs e) {
            if(lvEspList.SelectedItems.Count!=1) return;
            EspInfo ei=(EspInfo)lvEspList.SelectedItems[0].Tag;
            if(ei.BelongsTo!=EspInfo.UnknownOwner) {
                if(ei.BelongsTo==EspInfo.BaseOwner) {
                    MessageBox.Show("That plugin is an essental part of morrowind and cannot be removed", "Error");
                } else {
                    MessageBox.Show("That plugin is part of "+ei.BelongsTo+" and cannot be removed", "Error");
                }
                return;
            }
            if(Settings.WarnOnModDelete&&MessageBox.Show("This will delete "+ei.FileName+" permenently.\n"+
                "Are you sure you wish to do this?", "Warning", MessageBoxButtons.YesNo)!=DialogResult.Yes) {
                return;
            }
            File.Delete("data files\\"+ei.FileName);
            Program.Data.Esps.Remove(ei);
            lvEspList.Items.RemoveAt(lvEspList.SelectedIndices[0]);
        }

        private void cleanToolStripMenuItem_Click(object sender, EventArgs e) {
            if(lvModList.SelectedItems.Count!=1) return;
            mmod o=(mmod)lvModList.SelectedItems[0].Tag;
            if(o.Conflict==ConflictLevel.Active) {
                MessageBox.Show(o.ModName+" must be deactivated before it can be cleaned.", "Error");
                return;
            }
            if(Settings.WarnOnModDelete&&MessageBox.Show("If you clean this mod some files may be permenently deleted.\n"+
                "Are you sure you wish to continue?", "Warning", MessageBoxButtons.YesNo)!=DialogResult.Yes) {
                return;
            }
            int DeletedCount=0;
            int SkippedCount=0;
            int NotFoundCount=0;
            o.Clean(ref DeletedCount, ref SkippedCount, ref NotFoundCount);
            Conflicts.UpdateConflicts();
            UpdateEspList();
            UpdatemmodList();
            MessageBox.Show("Files deleted: "+DeletedCount.ToString()+"\n"+
                "Files skipped: "+SkippedCount.ToString()+"\n"+
                "Files not found: "+NotFoundCount.ToString(), "Message");
        }

        private void deleteToolStripMenuItem1_Click(object sender, EventArgs e) {
            if(lvModList.SelectedItems.Count!=1) return;
            mmod o=(mmod)lvModList.SelectedItems[0].Tag;
            if(o.Conflict==ConflictLevel.Active) {
                if(MessageBox.Show(o.ModName+" is activate, and deleting it will leave any associated files in your data directory\n"+
                    "Are you sure you wish to continue?", "Error", MessageBoxButtons.YesNo)!=DialogResult.Yes)
                    return;
                o.DeletionDeactivate();
            }
            if(MessageBox.Show("This will delete "+o.FileName+" permenently.\n"+
                "Are you sure you wish to do this?", "Warning", MessageBoxButtons.YesNo)!=DialogResult.Yes) {
                return;
            }
            o.Close();
            File.Delete(Settings.mmodDir+o.FileName);
            Program.Data.mmods.Remove(o);
            lvModList.Items.RemoveAt(lvModList.SelectedIndices[0]);
        }

        private void viewReadmeToolStripMenuItem_Click(object sender, EventArgs e) {
            if(lvModList.SelectedItems.Count!=1) return;
            mmod o=(mmod)lvModList.SelectedItems[0].Tag;
            try {
                string s=o.GetReadme();
                TextEditor te=new TextEditor("View readme", s, true, true);
                if(te.ShowDialog()==DialogResult.Yes) {
                    o.ReplaceReadme(te.Result);

                }
            } catch(Exception ex) {
                MessageBox.Show("Error opening readme.\n"+ex.Message, "Error");
            }
        }

        private void viewScriptToolStripMenuItem_Click(object sender, EventArgs e) {
            if(lvModList.SelectedItems.Count!=1) return;
            mmod o=(mmod)lvModList.SelectedItems[0].Tag;
            try {
                string s=o.GetScript();
                TextEditor te=new TextEditor("View script", s, false, true);
                if(te.ShowDialog()==DialogResult.Yes) {
                    o.ReplaceScript(te.Result);
                }
            } catch(Exception ex) {
                MessageBox.Show("Error opening script.\n"+ex.Message, "Error");
            }
        }

        private bool SwitchEsps(int index) {
            if(EspListSorter.order!=EspSortOrder.LoadOrder) {
                MessageBox.Show("Esps must be sorted by load order before you can switch them", "Error");
                return false;
            }
            EspInfo bottom=(EspInfo)lvEspList.Items[index].Tag;
            EspInfo top=(EspInfo)lvEspList.Items[index+1].Tag;
            if(bottom.MustLoadBefore.IndexOf(top.LowerFileName)!=-1) {
                MessageBox.Show(bottom.FileName+" must load before "+top.FileName, "Error");
                return false;
            }
            if(top.MustLoadAfter.IndexOf(bottom.LowerFileName)!=-1) {
                MessageBox.Show(top.FileName+" must load after "+bottom.FileName, "Error");
                return false;
            }
            DateTime oldtime=File.GetLastWriteTime("data files\\"+bottom.FileName);
            File.SetLastWriteTime("data files\\"+bottom.FileName, File.GetLastWriteTime("data files\\"+top.FileName));
            File.SetLastWriteTime("data files\\"+top.FileName, oldtime);
            Program.Data.SortEsps();
            ListViewItem lvi=lvEspList.Items[index];
            lvEspList.Items.RemoveAt(index);
            lvEspList.Items.Insert(index+1, lvi);
            return true;
        }

        private void bMoveUp_Click(object sender, EventArgs e) {
            if(lvEspList.SelectedItems.Count!=1) return;
            if(lvEspList.SelectedIndices[0]==0) return;
            SwitchEsps(lvEspList.SelectedIndices[0]-1);
        }

        private void moveToTopToolStripMenuItem_Click(object sender, EventArgs e) {
            if(lvEspList.SelectedItems.Count!=1) return;
            int position=lvEspList.SelectedIndices[0];
            while(position!=0) {
                if(!SwitchEsps(--position)) break;
            }
        }

        private void bMoveDown_Click(object sender, EventArgs e) {
            if(lvEspList.SelectedItems.Count!=1) return;
            if(lvEspList.SelectedIndices[0]==lvEspList.Items.Count-1) return;
            SwitchEsps(lvEspList.SelectedIndices[0]);
        }

        private void moveToBottomToolStripMenuItem_Click(object sender, EventArgs e) {
            if(lvEspList.SelectedItems.Count!=1) return;
            int position=lvEspList.SelectedIndices[0];
            while(position!=lvEspList.Items.Count-1) {
                if(!SwitchEsps(position++)) break;
            }
        }

        private void lvEspList_KeyDown(object sender, KeyEventArgs e) {
            if(e.Alt) {
                if(e.KeyCode==Keys.Up) {
                    bMoveUp_Click(null, null);
                    e.Handled=true;
                } else if(e.KeyCode==Keys.Down) {
                    bMoveDown_Click(null, null);
                    e.Handled=true;
                }
            }
        }

        private void lvModList_SelectedIndexChanged(object sender, EventArgs e) {
            if(lvModList.SelectedItems.Count!=1) return;
            mmod o=(mmod)lvModList.SelectedItems[0].Tag;
            //group filter
            foreach(ToolStripMenuItem tsmi in GroupToolStripMenuItem.DropDownItems) {
                if(tsmi==GroupNoneToolStripMenuItem) continue;
                if((o.group&(ulong)((ulong)0x01<<(int)tsmi.Tag))>0) {
                    tsmi.Checked=true;
                } else {
                    tsmi.Checked=false;
                }
            }
            //menus
            if(o.DoesReadmeExist()) viewReadmeToolStripMenuItem.Text="View readme";
            else viewReadmeToolStripMenuItem.Text="Create readme";
            if(o.DoesScriptExist()) viewScriptToolStripMenuItem.Text="View script";
            else viewScriptToolStripMenuItem.Text="Create script";
            if(o.Website=="") visitWebsiteToolStripMenuItem.Enabled=false;
            else visitWebsiteToolStripMenuItem.Enabled=true;
            if(o.Email=="") emailAuthorToolStripMenuItem.Enabled=false;
            else emailAuthorToolStripMenuItem.Enabled=true;
            //image
            pictureBox1.Image=o.image;
            //activation button
            switch(o.Conflict) {
            case ConflictLevel.Active:
                bActivate.Text="Deactivate";
                bActivate.Enabled=true;
                break;
            case ConflictLevel.Unusable:
                bActivate.Text="Activate";
                bActivate.Enabled=false;
                break;
            default:
                bActivate.Text="Activate";
                bActivate.Enabled=true;
                break;
            }
        }

        private void bActivate_Click(object sender, EventArgs e) {
            lvModList_ItemActivate(null, null);
        }

        private void bConflict_Click(object sender, EventArgs e) {
            //(new TextEditor("Conflict report", Conflicts.UpdateConflictsWithReport(), false, false)).ShowDialog();
            ConflictDetector.ReportGenerator.GenerateReport();
        }

        private void infoToolStripMenuItem_Click(object sender, EventArgs e) {
            if(lvModList.SelectedItems.Count!=1) return;
            mmod o=(mmod)lvModList.SelectedItems[0].Tag;
            string report=o.GetInfo();
            (new TextEditor(o.FileName+" info", report, false, false)).ShowDialog();
        }

        private void bLoad_Click(object sender, EventArgs e) {
            OpenDialog.Title="Select mod files to load";
            OpenDialog.Filter="morrowind mod file (*.mmod)|*.mmod";
            OpenDialog.Multiselect=true;
            if(OpenDialog.ShowDialog()==DialogResult.OK) {
                foreach(string s in OpenDialog.FileNames) Program.LoadNewmmod(s);
                UpdatemmodList();
            }
        }

        private void visitWebsiteToolStripMenuItem_Click(object sender, EventArgs e) {
            if(lvModList.SelectedItems.Count!=1) return;
            mmod o=(mmod)lvModList.SelectedItems[0].Tag;
            if(o.Website=="") {
                MessageBox.Show(o.FileName+" does not have a website", "Error");
                return;
            }
            string s=o.Website;
            if(!s.ToLower().StartsWith("http://")) s="http://"+s;
            System.Diagnostics.Process p=System.Diagnostics.Process.Start(s);
            if(p!=null) p.Close();
        }

        private void emailAuthorToolStripMenuItem_Click(object sender, EventArgs e) {
            if(lvModList.SelectedItems.Count!=1) return;
            mmod o=(mmod)lvModList.SelectedItems[0].Tag;
            if(o.Email=="") {
                MessageBox.Show(o.FileName+" does not have an email address", "Error");
                return;
            }
            string s="mailto:"+o.Email+"?subject="+o.FileName;
            System.Diagnostics.Process p=System.Diagnostics.Process.Start(s);
            if(p!=null) p.Close();
        }

        private void bHelp_Click(object sender, EventArgs e) {
            try {
                System.Diagnostics.Process.Start("mwmm\\mwmm.chm");
            } catch(Exception ex) {
                MessageBox.Show("An error occured while trying to open the help file.\n"+ex.Message, "Error");
            }
        }

        private void cmbEspSortOrder_SelectedIndexChanged(object sender, EventArgs e) {
            EspListSorter.order=(EspSortOrder)cmbEspSortOrder.SelectedIndex;
            if(EspListSorter.order!=EspSortOrder.LoadOrder) {
                bMoveDown.Enabled=false;
                bMoveUp.Enabled=false;
                moveDownToolStripMenuItem.Enabled=false;
                moveUpToolStripMenuItem.Enabled=false;
            } else {
                bMoveDown.Enabled=true;
                bMoveUp.Enabled=true;
                moveDownToolStripMenuItem.Enabled=true;
                moveUpToolStripMenuItem.Enabled=true;
            }
            lvEspList.Sort();
        }

        private void bAbout_Click(object sender, EventArgs e) {
            (new About()).ShowDialog();
            if(Program.Launch!=LaunchType.None) Close();
        }

        private void bLaunch_Click(object sender, EventArgs e) {
            Program.Launch=LaunchType.morrowind;
            Close();
        }

        private void bEdit_Click(object sender, EventArgs e) {
            if(lvModList.SelectedItems.Count!=1) return;
            pictureBox1.Image=null;
            if(CreateModForm.ShowForm((mmod)lvModList.SelectedItems[0].Tag)) {
                UpdatemmodList();
            }
        }

        private void pictureBox1_Click(object sender, EventArgs e) {
            if(pictureBox1.Image!=null) (new ImageForm(pictureBox1.Image)).ShowDialog();
        }

        private void TidyDataFolder() {
            System.Collections.Generic.List<DirectoryInfo> dis=new System.Collections.Generic.List<DirectoryInfo>();
            bool match=true;
            while(match) {
                dis.Clear();
                dis.Add(new DirectoryInfo("data files"));
                match=false;
                for(int i=0;i<dis.Count;i++) {
                    DirectoryInfo[] temp=dis[i].GetDirectories();
                    if(temp.Length!=0) {
                        dis.AddRange(temp);
                    } else {
                        if(dis[i].GetFileSystemInfos().Length==0) {
                            dis[i].Delete();
                            dis.RemoveAt(i--);
                            match=true;
                        }
                    }
                }
            }
        }

        private void MainForm_FormClosing(object sender, FormClosingEventArgs e) {
            TidyDataFolder();
            Settings.mfSize=Size;
            if(WindowState==FormWindowState.Maximized) Settings.mfMaximized=true;
            else Settings.mfMaximized=false;
            Settings.mfSplitterPos=splitContainer1.SplitterDistance;
            Settings.EspColWidth1=lvEspList.Columns[0].Width;
            Settings.EspColWidth2=lvEspList.Columns[1].Width;
            Settings.EspSortOrder=cmbEspSortOrder.SelectedIndex;
            Settings.mmodSortOrder=cmbmmodSortOrder.SelectedIndex;
        }

        private void bSettings_Click(object sender, EventArgs e) {
            (new OptionsForm()).ShowDialog();
            UpdateGroups();
        }

        private void IgnoreKeyPress(object sender, KeyPressEventArgs e) {
            e.Handled=true;
        }

        private void cmbmmodSortOrder_SelectedIndexChanged(object sender, EventArgs e) {
            mmodListSorter.order=(mmodSortOrder)cmbmmodSortOrder.SelectedIndex;
            lvModList.Sort();
        }

        private void cmbGroupFilter_SelectedIndexChanged(object sender, EventArgs e) {
            if(cmbGroupFilter.SelectedIndex==cmbGroupFilter.Items.Count-1) {
                cmbGroupFilter.SelectedIndex=0;
                bSettings_Click(null, null);
            } else {
                UpdatemmodList();
            }
        }

        private void UpdateGroups() {
            GroupToolStripMenuItem.DropDownItems.Clear();
            GroupToolStripMenuItem.DropDownItems.Add(GroupNoneToolStripMenuItem);
            for(int i=0;i<Settings.mmodGroups.Count;i++) {
                ToolStripMenuItem tsmi=new ToolStripMenuItem(Settings.mmodGroups[i]);
                tsmi.Tag=i;
                tsmi.Click+=GroupNoneToolStripMenuItem_Click;
                GroupToolStripMenuItem.DropDownItems.Add(tsmi);
            }
            cmbGroupFilter.Items.Clear();
            cmbGroupFilter.Items.Add("All");
            cmbGroupFilter.Items.Add("Unassigned");
            cmbGroupFilter.Items.AddRange(Settings.mmodGroups.ToArray());
            cmbGroupFilter.Items.Add("[Edit groups]");
            cmbGroupFilter.SelectedIndex=0;
        }

        private void GroupNoneToolStripMenuItem_Click(object _sender, EventArgs e) {
            if(lvModList.SelectedItems.Count!=1) return;
            ToolStripMenuItem sender=(ToolStripMenuItem)_sender;
            mmod o=(mmod)lvModList.SelectedItems[0].Tag;
            if(sender==GroupNoneToolStripMenuItem) {
                o.group=0;
                lvModList_SelectedIndexChanged(null, null);
                return;
            }
            sender.Checked=!sender.Checked;
            if(sender.Checked) {
                o.group|=(ulong)((ulong)1<<(int)sender.Tag);
            } else {
                o.group&=0xFFFFFFFFFFFFFFFF - (ulong)((ulong)1<<(int)sender.Tag);
            }
        }

        private void forceDisableToolStripMenuItem_Click(object sender, EventArgs e) {
            if(lvModList.SelectedItems.Count!=1) return;
            mmod o=(mmod)lvModList.SelectedItems[0].Tag;
            if(o.Conflict!=ConflictLevel.Active) o.Deactivate(true);
        }

        private void viewDataConflictsToolStripMenuItem_Click(object sender, EventArgs e) {
            if(lvModList.SelectedItems.Count!=1) return;
            mmod o=(mmod)lvModList.SelectedItems[0].Tag;
            (new TextEditor("data file and script conflicts", Conflicts.ConflictReport(o), false, false)).ShowDialog();
        }

        private void convertToArchiveToolStripMenuItem_Click(object sender, EventArgs e) {
            SavemmodDialog.Title="Save mmod as";
            SavemmodDialog.Filter="7-zip (*.7z)|*.7z|zip (*.zip)|*.zip";
            SavemmodDialog.AddExtension=false;
            SavemmodDialog.OverwritePrompt=false;
            try {
                if(lvModList.SelectedItems.Count!=1) return;
                mmod o=(mmod)lvModList.SelectedItems[0].Tag;
                SavemmodDialog.FileName=Path.GetFileNameWithoutExtension(o.FileName);
                if(SavemmodDialog.ShowDialog()!=DialogResult.OK) return;
                string ZipName=SavemmodDialog.FileName;
                if(SavemmodDialog.FilterIndex==1) {
                    if(Path.GetExtension(ZipName).ToLower()!=".7z") ZipName+=".7z";
                } else {
                    if(Path.GetExtension(ZipName).ToLower()!=".zip") ZipName+=".zip";
                }
                if(File.Exists(ZipName)&&MessageBox.Show("File '"+ZipName+"' already exists.\n"+
                    "Do you wish to overwrite?", "Warning", MessageBoxButtons.YesNo)!=DialogResult.Yes) return;
                string path=o.GetDataFiles();
                string temppath;
                if(path==null) path=Program.CreateTempDirectory();
                if(o.DoesReadmeExist()) {
                    File.WriteAllText(path+o.ModName+"_readme.rtf", o.GetReadme());
                }
                Directory.CreateDirectory(path+Program.mmodConversionData);
                if(o.DoesScriptExist()) {
                    File.WriteAllText(path+Program.mmodConversionData+"script.txt", o.GetScript());
                }
                File.WriteAllText(path+Program.mmodConversionData+"info.txt", o.GetInfo());
                temppath=o.GetImage();
                if(temppath!=null) {
                    System.Drawing.Imaging.ImageFormat imf=o.image.RawFormat;
                    File.Move(temppath, path+Program.mmodConversionData+"screenshot");
                }
                temppath=o.GetConfig();
                File.Move(temppath, path+Program.mmodConversionData+"config");
                temppath=o.GetPlugins();
                if(temppath!=null) {
                    foreach(string s in Directory.GetFiles(temppath)) {
                        File.Move(s, path+Path.GetFileName(s));
                    }
                }
                if(SavemmodDialog.FilterIndex==1) {
                    System.Diagnostics.ProcessStartInfo psi=new System.Diagnostics.ProcessStartInfo("mwmm\\7za.exe");
                    psi.Arguments="a -t7z -mx=9 \""+ZipName+"\" \""+path+"*\"";
                    psi.CreateNoWindow=true;
                    psi.UseShellExecute=false;
                    System.Diagnostics.Process p=System.Diagnostics.Process.Start(psi);
                    p.WaitForExit();
                } else {
                    ICSharpCode.SharpZipLib.Zip.FastZip fs=new ICSharpCode.SharpZipLib.Zip.FastZip();
                    fs.CreateZip(ZipName, path, true, null);
                }
            } catch(Exception ex) {
                MessageBox.Show("Conversion failed\nError: "+ex.Message, "Error");
                return;
            }
            MessageBox.Show("Conversion sucessful", "Message");
        }

        private void PipeFileWatcher_Created(object sender, FileSystemEventArgs e) {
            System.Threading.Thread.Sleep(100); //Give the other process a chance to close the file
            if(Application.OpenForms.Count>1) {
                MessageBox.Show("You can only load new mmods when mwmm is displaying the data file list", "Error");
            } else {
                string[] mods=File.ReadAllLines("mwmm\\pipe");
                foreach(string s in mods) Program.LoadNewmmod(s);
                UpdatemmodList();
            }
            File.Delete("mwmm\\pipe");
        }

        private void mmodReportToolStripMenuItem_Click(object sender, EventArgs e) {
            SavemmodDialog.Title="Save report as";
            SavemmodDialog.Filter="text file|*.txt";
            SavemmodDialog.AddExtension=true;
            SavemmodDialog.OverwritePrompt=true;
            if(SavemmodDialog.ShowDialog()==DialogResult.OK) {
                StreamWriter sw=new StreamWriter(SavemmodDialog.FileName);
                char t='\t';
                foreach(mmod o in Program.Data.mmods) {
                    sw.WriteLine(o.FileName+t+o.ModName+t+o.Version+t+
                        o.Author+t+o.Email+t+o.Website+t+o.CreationTime.ToString());
                }
                sw.Close();
            }
        }

        private void Activatemmod(mmod mod, bool force) {
            if(mod.Conflict!=ConflictLevel.Active) Conflicts.UpdateConflict(mod);
            bool warn=!force;
            switch(mod.Conflict) {
            case ConflictLevel.Active:
                if(!force) {
                    if(!Settings.WarnOnModDelete||MessageBox.Show("Deactivate "+mod.FileName+"?", "Question", MessageBoxButtons.YesNo)==DialogResult.Yes) {
                        mod.Deactivate(false);
                    }
                }
                return;
            case ConflictLevel.MajorConflict:
                if(!force) {
                    switch(MessageBox.Show(mod.FileName+" conflicts with an already active mod.\n"+
                    "Do you wish to be warned about any file conflicts?", "Warning", MessageBoxButtons.YesNoCancel)) {
                    case DialogResult.Yes: warn=true; break;
                    case DialogResult.No: warn=false; break;
                    case DialogResult.Cancel: return;
                    }
                }
                break;
            case ConflictLevel.Unusable:
                if(!force) MessageBox.Show(mod.FileName+" is unusable due to conflicts.", "Error");
                return;
            }
            try {
                mod.Activate(warn);
            } catch(Exception ex) {
                MessageBox.Show("An error occured while activating the mod.\n"+
                    "It may not have been activated completely.\n"+
                    "Error: "+ex.Message, "Error");
            }
        }

        private void activateAllToolStripMenuItem_Click(object sender, EventArgs e) {
            foreach(ListViewItem lvi in lvModList.Items) {
                Activatemmod((mmod)lvi.Tag, true);
            }
            RefreshLists();
        }

        private void deactivateAllToolStripMenuItem_Click(object sender, EventArgs e) {
            foreach(ListViewItem lvi in lvModList.Items) {
                ((mmod)lvi.Tag).Deactivate(true);
            }
            RefreshLists();
        }

        private void activateAllToolStripMenuItem1_Click(object sender, EventArgs e) {
            foreach(ListViewItem lvi in lvEspList.Items) {
                ((EspInfo)lvi.Tag).Active=true;
                lvi.Checked=true;
            }
            UpdateEspList();
        }

        private void deactivateAllToolStripMenuItem1_Click(object sender, EventArgs e) {
            foreach(ListViewItem lvi in lvEspList.Items) {
                if(((EspInfo)lvi.Tag).BelongsTo==EspInfo.BaseOwner) continue;
                if(((EspInfo)lvi.Tag).Deactivatable==DeactiveStatus.Disallow) continue;
                ((EspInfo)lvi.Tag).Active=false;
                lvi.Checked=false;
            }
        }

        private void cleanAllToolStripMenuItem_Click(object sender, EventArgs e) {
            if(MessageBox.Show("This option will disable all mods and permenently delete all data files and esps which are linked to mmods.\n"+
                "Are you sure you wish to continue?", "Warning", MessageBoxButtons.YesNo)!=DialogResult.Yes) {
                return;
            }
            //Deactivate everything
            foreach(mmod o in Program.Data.mmods) o.Deactivate(true);
            //Clear up any left over mwmm stuff
            Program.Data.BSAs.Clear();
            Program.Data.DataFiles.Clear();
            Program.Data.INIEdits.Clear();
            for(int i=0;i<Program.Data.Esps.Count;i++) {
                if(Program.Data.Esps[i].Parent!=null) Program.Data.Esps.RemoveAt(i--);
            }
            //Clear out the data folder
            foreach(mmod o in Program.Data.mmods) o.Clean();
            TidyDataFolder();
            RefreshLists();
        }

        private void cleanFilteredToolStripMenuItem_Click(object sender, EventArgs e) {
            int DeletedCount=0;
            int SkippedCount=0;
            int NotFoundCount=0;
            foreach(ListViewItem lvi in lvModList.Items) {
                ((mmod)lvi.Tag).Clean(ref DeletedCount, ref SkippedCount, ref NotFoundCount);
            }
            RefreshLists();
            MessageBox.Show("Files deleted: "+DeletedCount.ToString()+"\n"+
                "Files skipped: "+SkippedCount.ToString()+"\n"+
                "Files not found: "+NotFoundCount.ToString(), "Message");
        }

        private void tidyDataFolderToolStripMenuItem_Click(object sender, EventArgs e) { TidyDataFolder(); }

        private void bBatch_Click(object sender, EventArgs e) {
            BatchContextMenu.Show(bBatch, 0, 0);
        }

        private void bImport_Click(object sender, EventArgs e) {
            ImportContextMenu.Show(bImport, 0, 0);
        }

        private void exportmmodListToolStripMenuItem_Click(object sender, EventArgs e) {
            SavemmodDialog.Title="Save active mmod list as";
            SavemmodDialog.Filter="Active mmod list (*.oaml)|*.oaml";
            SavemmodDialog.AddExtension=false;
            SavemmodDialog.OverwritePrompt=false;
            if(SavemmodDialog.ShowDialog()!=DialogResult.OK) return;
            if(Path.GetExtension(SavemmodDialog.FileName).ToLower()!=".oaml") SavemmodDialog.FileName+=".oaml";
            if(File.Exists(SavemmodDialog.FileName)&&MessageBox.Show("File '"+SavemmodDialog.FileName+"' already exists.\n"+
                    "Do you wish to overwrite?", "Warning", MessageBoxButtons.YesNo)!=DialogResult.Yes) return;
            BinaryWriter bw=new BinaryWriter(File.Create(SavemmodDialog.FileName));
            foreach(mmod o in Program.Data.mmods) {
                if(o.Conflict==ConflictLevel.Active) bw.Write(o.CRC);
            }
            bw.Close();
        }

        private void exportLoadOrderToolStripMenuItem_Click(object sender, EventArgs e) {
            SavemmodDialog.Title="Save load order data as";
            SavemmodDialog.Filter="Load order data (*.olod)|*.olod";
            SavemmodDialog.AddExtension=false;
            SavemmodDialog.OverwritePrompt=false;
            if(SavemmodDialog.ShowDialog()!=DialogResult.OK) return;
            if(Path.GetExtension(SavemmodDialog.FileName).ToLower()!=".olod") SavemmodDialog.FileName+=".olod";
            if(File.Exists(SavemmodDialog.FileName)&&MessageBox.Show("File '"+SavemmodDialog.FileName+"' already exists.\n"+
                    "Do you wish to overwrite?", "Warning", MessageBoxButtons.YesNo)!=DialogResult.Yes) return;
            BinaryWriter bw=new BinaryWriter(File.Create(SavemmodDialog.FileName));
            foreach(EspInfo ei in Program.Data.Esps) {
                bw.Write(ei.LowerFileName);
                bw.Write(ei.DateModified.ToBinary());
            }
            bw.Close();
        }

        private void importmmodListToolStripMenuItem_Click(object sender, EventArgs e) {
            OpenDialog.Title="Select active mmod list to import";
            OpenDialog.Filter="Active mmod list (*.oaml)|*.oaml";
            OpenDialog.Multiselect=false;
            if(OpenDialog.ShowDialog()!=DialogResult.OK) return;
            BinaryReader br=new BinaryReader(File.OpenRead(OpenDialog.FileName));
            uint[] crcs=new uint[br.BaseStream.Length/4];
            int i=0;
            while(br.BaseStream.Position<br.BaseStream.Length) {
                crcs[i++]=br.ReadUInt32();
            }
            br.Close();
            foreach(mmod o in Program.Data.mmods) {
                if(o.Conflict==ConflictLevel.Active) {
                    if(Array.IndexOf<uint>(crcs, o.CRC)==-1) o.Deactivate(true);
                }
            }
            foreach(mmod o in Program.Data.mmods) {
                if(o.Conflict!=ConflictLevel.Active) {
                    try {
                        if(Array.IndexOf<uint>(crcs, o.CRC)!=-1) o.Activate(false);
                    } catch {
                        MessageBox.Show("An error occured activating "+o.FileName, "Error");
                    }
                }
            }
            RefreshLists();
        }

        private void importLoadOrderToolStripMenuItem_Click(object sender, EventArgs e) {
            OpenDialog.Title="Select load order data file to import";
            OpenDialog.Filter="Load order data (*.olod)|*.olod";
            OpenDialog.Multiselect=false;
            if(OpenDialog.ShowDialog()!=DialogResult.OK) return;
            BinaryReader br=new BinaryReader(File.OpenRead(OpenDialog.FileName));
            while(br.BaseStream.Position<br.BaseStream.Length) {
                string file=br.ReadString();
                DateTime pos=DateTime.FromBinary(br.ReadInt64());
                EspInfo ei=Program.Data.GetEsp(file);
                if(ei!=null) {
                    ei.DateModified=pos;
                    FileInfo fi=new FileInfo("data files\\"+file);
                    fi.LastWriteTime=pos;
                } else {
                    MessageBox.Show("esp '"+file+"' was not found", "Error");
                }
            }
            br.Close();
            Program.Data.SortEsps();
            UpdateEspList();
        }

        private void deleteBackupsToolStripMenuItem_Click(object sender, EventArgs e) {
            bool error=false;
            foreach(string s in Directory.GetFiles(Program.BackupDir, "*.mmod")) {
                try { File.Delete(s); } catch { error=true; }
            }
            foreach(string s in Directory.GetFiles(Program.CorruptDir, "*.mmod")) {
                try { File.Delete(s); } catch { error=true; }
            }
            if(error) {
                MessageBox.Show("An error occured, and one or more files could not be deleted.\n"+
                    "Make sure that none of the files in the backup or corrupt folders are write protected.",
                    "Error");
            }
        }

        private void editToolStripMenuItem_Click(object sender, EventArgs e) {
            bEdit_Click(null, null);
        }

        private void updateConflictsToolStripMenuItem_Click(object sender, EventArgs e) {
            Conflicts.UpdateConflicts();
            UpdateEspList();
            UpdatemmodList();
        }
    }
}