using System;
using System.Windows.Forms;

namespace MorrowindModManager {
    public partial class OptionsForm : Form {
        public OptionsForm() {
            InitializeComponent();
            cbWarnings.Checked=Settings.ShowScriptWarnings;
            cbIniWarn.Checked=Settings.WarnOnINIEdit;
            cbDataWarnings.Checked=Settings.WarnOnModDelete;
            cbmmodInfo.Checked=Settings.ShowNewModInfo;
            cbLockFOV.Checked=Settings.LockFOV;
            cbShowEspWarnings.Checked=Convert.ToBoolean((byte)Settings.DefaultEspWarn);
            tbCommandLine.Text=Settings.morrowindCommandLine;
            cmbGroups.Items.AddRange(Settings.mmodGroups.ToArray());
            cbCompressionBoost.Checked=Settings.CompressionBoost;
            cbAutoupdateConflicts.Checked=Settings.AutoUpdateConflicts;
        }

        private int SelectedGroup=0;

        private void cbWarnings_CheckedChanged(object sender, EventArgs e) {
            Settings.ShowScriptWarnings=cbWarnings.Checked;
        }

        private void cmbGroups_SelectedIndexChanged(object sender, EventArgs e) {
            SelectedGroup=cmbGroups.SelectedIndex;
            bAdd.Enabled=false;
            bRenameGroup.Enabled=false;
            if(cmbGroups.SelectedIndex!=0) {
                bDelete.Enabled=true;
                bMoveToEnd.Enabled=true;
            } else {
                bDelete.Enabled=false;
                bMoveToEnd.Enabled=false;
            }
        }

        private void cmbGroups_TextChanged(object sender, EventArgs e) {
            bAdd.Enabled=true;
            if(SelectedGroup!=0) bRenameGroup.Enabled=true;
            bDelete.Enabled=false;
            bMoveToEnd.Enabled=false;
        }

        private void bMoveToEnd_Click(object sender, EventArgs e) {
            bool[] bools=new bool[Program.Data.mmods.Count];
            int index=cmbGroups.SelectedIndex-1;
            for(int i=0;i<Program.Data.mmods.Count;i++) {
                if((Program.Data.mmods[i].group&(ulong)((ulong)1<<index))>0) bools[i]=true;
                else bools[i]=false;
            }
            string name=cmbGroups.Text;
            bDelete_Click(null, null);
            cmbGroups.Text=name;
            bAdd_Click(null, null);
            index=cmbGroups.Items.Count-2;
            for(int i=0;i<Program.Data.mmods.Count;i++) {
                if(bools[i]) {
                    Program.Data.mmods[i].group|=(ulong)((ulong)1<<index);
                } else {
                    Program.Data.mmods[i].group&=0xFFFFFFFFFFFFFFFF - (ulong)((ulong)1<<index);
                }
            }
        }

        private void bDelete_Click(object sender, EventArgs e) {
            int i=cmbGroups.SelectedIndex;
            if(i==-1) return;
            cmbGroups.Items.RemoveAt(i);
            Settings.mmodGroups.RemoveAt(i-1);
            bDelete.Enabled=false;
            ulong bit=(ulong)(1<<i);
            foreach(mmod o in Program.Data.mmods) {
                ulong n=0;
                for(int i2=0;i2<i-1;i2++) {
                    if((o.group&(ulong)((ulong)1<<i2))>0) {
                        n|=(ulong)((ulong)1<<i2);
                    }
                }
                for(int i2=i;i2<64;i2++) {
                    if((o.group&(ulong)((ulong)1<<i2))>0) {
                        n|=(ulong)((ulong)1<<(i2-1));
                    }
                }
                o.group=n;
            }
        }

        private void bAdd_Click(object sender, EventArgs e) {
            if(cmbGroups.Items.Count>=60) {
                MessageBox.Show("You can only create a maximum of 60 groups", "Error");
                return;
            }
            if(cmbGroups.Items.Contains(cmbGroups.Text)) {
                MessageBox.Show("A group with that name already exists","Error");
                return;
            }
            cmbGroups.Items.Add(cmbGroups.Text);
            Settings.mmodGroups.Add(cmbGroups.Text);
            bAdd.Enabled=false;
        }

        private void bRenameGroup_Click(object sender, EventArgs e) {
            cmbGroups.Items[SelectedGroup]=cmbGroups.Text;
            Settings.mmodGroups[SelectedGroup-1]=cmbGroups.Text;
            bRenameGroup.Enabled=false;
        }

        private void bClearGroups_Click(object sender, EventArgs e) {
            cmbGroups.SelectedIndex=0;
            cmbGroups.Items.Clear();
            cmbGroups.Items.Add("None");
            Settings.mmodGroups.Clear();
            foreach(mmod o in Program.Data.mmods) {
                o.group=0;
            }

        }

        private void cbIniWarn_CheckedChanged(object sender, EventArgs e) {
            Settings.WarnOnINIEdit=cbIniWarn.Checked;
        }

        private void cbDataWarnings_CheckedChanged(object sender, EventArgs e) {
            Settings.WarnOnModDelete=cbDataWarnings.Checked;
        }

        private void cbmmodInfo_CheckedChanged(object sender, EventArgs e) {
            Settings.ShowNewModInfo=cbmmodInfo.Checked;
        }

        private void cbLockFOV_CheckedChanged(object sender, EventArgs e) {
            Settings.LockFOV=cbLockFOV.Checked;
        }

        private void bMoveModFolder_Click(object sender, EventArgs e) {
            if(MessageBox.Show("Moving the mmod dir is not recommended for normal users.\n"+
                "Please ensure you have sufficent space in the new folder to move your whole mod collection.\n"+
                "Do you wish to continue?", "Warning", MessageBoxButtons.YesNo)!=DialogResult.Yes) return;
            if(mmodDirDialog.ShowDialog()==DialogResult.OK) {
                foreach(string s in System.IO.Directory.GetFiles(Settings.mmodDir,"*.mmod")) {
                    foreach(mmod o in Program.Data.mmods) {
                        o.Close();
                    }
                    System.IO.File.Move(s, mmodDirDialog.SelectedPath+"\\"+System.IO.Path.GetFileName(s));
                }
                Settings.mmodDir=mmodDirDialog.SelectedPath.ToLower();
                if(!Settings.mmodDir.EndsWith("\\")) Settings.mmodDir+="\\";
            }
        }

        private void cbShowEspWarnings_CheckedChanged(object sender, EventArgs e) {
            if(cbShowEspWarnings.Checked) {
                Settings.DefaultEspWarn=DeactiveStatus.WarnAgainst;
            } else {
                Settings.DefaultEspWarn=DeactiveStatus.Allow;
            }
        }

        private void tbCommandLine_TextChanged(object sender, EventArgs e) {
            Settings.morrowindCommandLine=tbCommandLine.Text;
        }

        private void cmbArchiveMode_KeyPress(object sender, KeyPressEventArgs e) {
            e.Handled=true;
        }

        private void cbCompressionBoost_CheckedChanged(object sender, EventArgs e) {
            Settings.CompressionBoost=cbCompressionBoost.Checked;
        }

        private void cbAutoupdateConflicts_CheckedChanged(object sender, EventArgs e) {
            Settings.AutoUpdateConflicts=cbAutoupdateConflicts.Checked;
        }

    }
}