using System;
using Application=System.Windows.Forms.Application;
using MessageBox=System.Windows.Forms.MessageBox;
using System.IO;
using Formatter=System.Runtime.Serialization.Formatters.Binary.BinaryFormatter;
using System.Collections.Generic;
using Mutex=System.Threading.Mutex;

namespace MorrowindModManager {
    
    [Serializable]
    public class sData {
        public const int CurrentVersion=4;

        public readonly List<mmod> mmods=new List<mmod>();

        public readonly List<DataFileInfo> DataFiles=new List<DataFileInfo>();

        public readonly List<EspInfo> Esps=new List<EspInfo>();

        public readonly List<BSA> BSAs=new List<BSA>();

        public readonly List<INIEditInfo> INIEdits=new List<INIEditInfo>();

        public bool DoesModExist(string s) {
            s=s.ToLower();
            foreach(mmod o in mmods) {
                if(o.LowerFileName==s) return true;
            }
            return false;
        }
        public bool DoesModExist(ConflictData cd,bool Active) {
            foreach(mmod o in mmods) {
                if(Active&&o.Conflict!=ConflictLevel.Active) continue;
                if(cd==o) return true;
            }
            return false;
        }

        public mmod GetMod(string s) {
            s=s.ToLower();
            foreach(mmod o in mmods) {
                if(o.LowerFileName==s) return o;
            }
            return null;
        }

        public bool DoesEspExist(string s) {
            s=s.ToLower();
            foreach(EspInfo ei in Esps) {
                if(ei.LowerFileName==s) return true;
            }
            return false;
        }

        public EspInfo GetEsp(string s) {
            s=s.ToLower();
            foreach(EspInfo ei in Esps) {
                if(ei.LowerFileName==s) return ei;
            }
            return null;
        }

        public bool DoesDataFileExist(string s) {
            s=s.ToLower();
            foreach(DataFileInfo sfi in DataFiles) {
                if(sfi.LowerFileName==s) return true;
            }
            return false;
        }
        public bool DoesDataFileExist(DataFileInfo dfi) {
            foreach(DataFileInfo sfi in DataFiles) {
                if(sfi==dfi) return true;
            }
            return false;
        }

        public DataFileInfo GetDataFile(string s) {
            s=s.ToLower();
            foreach(DataFileInfo sfi in DataFiles) {
                if(sfi.LowerFileName==s) return sfi;
            }
            return null;
        }
        public DataFileInfo GetDataFile(DataFileInfo dfi) {
            foreach(DataFileInfo sfi in DataFiles) {
                if(sfi==dfi) return sfi;
            }
            return null;
        }

        public bool DoesBSAExist(string s) {
            s=s.ToLower();
            foreach(BSA b in BSAs) {
                if(b.LowerFileName==s) return true;
            }
            return false;
        }

        public BSA GetBSA(string s) {
            s=s.ToLower();
            foreach(BSA b in BSAs) {
                if(b.LowerFileName==s) return b;
            }
            return null;
        }

        public void SortEsps() {
            foreach(EspInfo ei in Esps) {
                ei.DateModified=File.GetLastWriteTime("data files\\"+ei.FileName);
            }
            bool moved=true;
            while(moved) {
                moved=false;
                for(int i=0;i<Esps.Count-1;i++) {
                    if(Esps[i].DateModified>Esps[i+1].DateModified) {
                        EspInfo o=Esps[i+1];
                        Esps[i+1]=Esps[i];
                        Esps[i]=o;
                        moved=true;
                    }
                }
            }
        }

        public void InsertESP(EspInfo ei, PluginLoadInfo[] plis) {
            DateTime MinTime;
            DateTime MaxTime;
            DateTime InsertAt=DateTime.Now;
            MaxTime=File.GetLastWriteTime("data files\\"+((EspInfo)Esps[Esps.Count-1]).FileName);
            if(InsertAt-TimeSpan.FromMinutes(1)<=MaxTime) InsertAt=MaxTime+TimeSpan.FromMinutes(1);
            MinTime=DateTime.MinValue;
            MaxTime=DateTime.MaxValue;
            foreach(EspInfo esp in Esps) {
                foreach(string target in esp.MustLoadAfter) {
                    if(target!=ei.LowerFileName) continue;
                    DateTime temp=File.GetLastWriteTime("data files\\"+esp.FileName);
                    if(temp<MaxTime) MaxTime=temp;
                }
                foreach(string target in esp.MustLoadBefore) {
                    if(target!=ei.LowerFileName) continue;
                    DateTime temp=File.GetLastWriteTime("data files\\"+esp.FileName);
                    if(temp>MinTime) MinTime=temp;
                }
            }
            foreach(PluginLoadInfo pli in plis) {
                if(pli.Plugin.ToLower()!=ei.LowerFileName) continue;
                EspInfo target=GetEsp(pli.Target);
                if(pli.LoadAfter) {
                    ei.MustLoadAfter.Add(pli.Target.ToLower());
                    if(target!=null) {
                        DateTime temp=File.GetLastWriteTime("data files\\"+target.FileName);
                        if(temp>MinTime) MinTime=temp;
                    }
                } else {
                    ei.MustLoadBefore.Add(pli.Target.ToLower());
                    if(target!=null) {
                        DateTime temp=File.GetLastWriteTime("data files\\"+target.FileName);
                        if(temp<MaxTime) MaxTime=temp;
                    }
                }
            }
            if(MinTime>DateTime.MinValue&&MaxTime<DateTime.MaxValue&&MinTime>=MaxTime+TimeSpan.FromSeconds(2)) {
                MessageBox.Show("Unable to correctly insert plugin "+ei.FileName+" because of conflicting load order information.\n"+
                    "This is most likely to occur if two existing mods are already loading in the wrong order.\n"+
                    "Please check the readmes of other mods to ensure you have installed them correctly.\n"+
                    ei.FileName+" has been added to the end of the load order.", "Error");
            } else {
                if(InsertAt>MaxTime) InsertAt=MaxTime-TimeSpan.FromMinutes(2);
                while(InsertAt<MinTime+TimeSpan.FromSeconds(1)) InsertAt+=TimeSpan.FromSeconds(1);
                File.SetLastWriteTime("data files\\"+ei.FileName, InsertAt);
            }
            ei.Active=true;
            Esps.Add(ei);
            SortEsps();
        }
    }

    public static class Settings {
        public const int CurrentVersion = 3;

        public static int MaxMemoryStreamSize; //64 mb
        public static bool ShowScriptWarnings;
        public static System.Drawing.Size mfSize;
        public static bool mfMaximized;
        public static int mfSplitterPos;
        public static List<string> mmodGroups;
        public static bool WarnOnINIEdit;
        public static bool WarnOnModDelete;
        public static bool ShowNewModInfo;
        public static bool LockFOV;
        public static DeactiveStatus DefaultEspWarn;
        public static string morrowindCommandLine;
        public static int EspColWidth1;
        public static int EspColWidth2;
        public static System.Drawing.Size TextEdSize;
        public static bool TextEdMaximized;
        public static string mmodDir=@"mwmm\mods\";
        public static string FolderBrowserDir;
        public static bool AutoUpdateConflicts;

        public static bool CDShowMajor;
        public static bool CDShowMinor;
        public static bool CDShowVeryMinor;
        public static bool CDIncludeEsp;
        public static bool CDIgnoreInactiveEsp;
        public static bool CDIncludemmod;
        public static bool CDIgnoreInactivemmod;

        public static int EspSortOrder;
        public static int mmodSortOrder;

        public static bool UpdateInvalidation;
        public static ArchiveInvalidationFlags InvalidationFlags;

        public static bool CompressionBoost;
        public static CompressionType dataCompressionType;
        public static CompressionLevel mmodCompressionLevel;
        public static CompressionLevel dataCompressionLevel;

        public static void SaveSettings() {
            BinaryWriter bw=new BinaryWriter(File.Open(Program.SettingsFile, FileMode.Create));
            bw.Write(CurrentVersion);
            bw.Write(MaxMemoryStreamSize);
            bw.Write(ShowScriptWarnings);
            bw.Write(mfSize.Width);
            bw.Write(mfSize.Height);
            bw.Write(mfMaximized);
            bw.Write(mfSplitterPos);
            bw.Write((byte)mmodGroups.Count);
            foreach(string s in mmodGroups) bw.Write(s);
            bw.Write(UpdateInvalidation);
            bw.Write(WarnOnINIEdit);
            bw.Write(WarnOnModDelete);
            bw.Write(TextEdSize.Width);
            bw.Write(TextEdSize.Height);
            bw.Write(TextEdMaximized);
            bw.Write(ShowNewModInfo);
            bw.Write(LockFOV);
            bw.Write(mmodDir);
            bw.Write((byte)DefaultEspWarn);
            bw.Write(morrowindCommandLine);
            bw.Write(EspColWidth1);
            bw.Write(EspColWidth2);
            bw.Write(AutoUpdateConflicts);
            bw.Write(CDShowMajor);
            bw.Write(CDShowMinor);
            bw.Write(CDShowVeryMinor);
            bw.Write(CDIncludeEsp);
            bw.Write(CDIgnoreInactiveEsp);
            bw.Write(CDIncludemmod);
            bw.Write(CDIgnoreInactivemmod);
            bw.Write(EspSortOrder);
            bw.Write(mmodSortOrder);
            bw.Write(CompressionBoost);
            bw.Write((byte)dataCompressionType);
            bw.Write((byte)mmodCompressionLevel);
            bw.Write((byte)dataCompressionLevel);
            bw.Write(FolderBrowserDir);
            bw.Write((uint)InvalidationFlags);
            bw.Close();
        }
        public static void LoadSettings() {
            DefaultSettings();
            if(!File.Exists(Program.SettingsFile)) return;
            BinaryReader br;
            try {
                br=new BinaryReader(File.OpenRead(Program.SettingsFile));
            } catch { return; }
            try {
                int version=br.ReadInt32();
                MaxMemoryStreamSize=br.ReadInt32();
                ShowScriptWarnings=br.ReadBoolean();
                mfSize.Width=br.ReadInt32();
                mfSize.Height=br.ReadInt32();
                mfMaximized=br.ReadBoolean();
                mfSplitterPos=br.ReadInt32();
                byte b=br.ReadByte();
                for(int i=0;i<b;i++) mmodGroups.Add(br.ReadString());
                UpdateInvalidation=br.ReadBoolean();
                WarnOnINIEdit=br.ReadBoolean();
                WarnOnModDelete=br.ReadBoolean();
                TextEdSize.Width=br.ReadInt32();
                TextEdSize.Height=br.ReadInt32();
                TextEdMaximized=br.ReadBoolean();
                ShowNewModInfo=br.ReadBoolean();
                LockFOV=br.ReadBoolean();
                mmodDir=br.ReadString();
                DefaultEspWarn=(DeactiveStatus)br.ReadByte();
                morrowindCommandLine=br.ReadString();
                EspColWidth1=br.ReadInt32();
                EspColWidth2=br.ReadInt32();
                if(version>=2) {
                    AutoUpdateConflicts=br.ReadBoolean();
                }
                CDShowMajor=br.ReadBoolean();
                CDShowMinor=br.ReadBoolean();
                CDShowVeryMinor=br.ReadBoolean();
                CDIncludeEsp=br.ReadBoolean();
                CDIgnoreInactiveEsp=br.ReadBoolean();
                CDIncludemmod=br.ReadBoolean();
                CDIgnoreInactivemmod=br.ReadBoolean();
                EspSortOrder=br.ReadInt32();
                mmodSortOrder=br.ReadInt32();
                CompressionBoost=br.ReadBoolean();
                if(version>=3) {
                    dataCompressionType=(CompressionType)br.ReadByte();
                    mmodCompressionLevel=(CompressionLevel)br.ReadByte();
                    dataCompressionLevel=(CompressionLevel)br.ReadByte();
                }
                FolderBrowserDir=br.ReadString();
                InvalidationFlags=(ArchiveInvalidationFlags)br.ReadUInt32();
            } catch {
                DefaultSettings();
            } finally {
                if(br!=null) br.Close();
            }
        }
        public static void DefaultSettings() {
            MaxMemoryStreamSize=67108864; //64 mb
            ShowScriptWarnings=true;
            mfSize=System.Drawing.Size.Empty;
            mfMaximized=false;
            mfSplitterPos=0;
            mmodGroups=new List<string>();
            UpdateInvalidation=false;
            WarnOnINIEdit=true;
            WarnOnModDelete=true;
            ShowNewModInfo=false;
            LockFOV=false;
            DefaultEspWarn=DeactiveStatus.WarnAgainst;
            morrowindCommandLine="";
            EspColWidth1=-1;
            EspColWidth2=-1;
            InvalidationFlags=ArchiveInvalidationFlags.Default;
            TextEdSize=System.Drawing.Size.Empty;
            TextEdMaximized=false;
            AutoUpdateConflicts=true;

            CDShowMajor=true;
            CDShowMinor=false;
            CDShowVeryMinor=false;
            CDIncludeEsp=true;
            CDIgnoreInactiveEsp=false;
            CDIncludemmod=true;
            CDIgnoreInactivemmod=false;
            EspSortOrder=0;
            mmodSortOrder=0;
            CompressionBoost=false;
            dataCompressionType=CompressionType.SevenZip;
            mmodCompressionLevel=CompressionLevel.VeryHigh;
            dataCompressionLevel=CompressionLevel.High;
            FolderBrowserDir="";
        }
    }

    public enum LaunchType { None, morrowind, Updater };

    public static class Program {
        public const string version="0.8.4";
        public const byte MajorVersion=0;
        public const byte MinorVersion=8;
        public const byte BuildNumber=4;
        public const byte CurrentmmodVersion=2;

        public const string CorruptDir=@"mwmm\corrupt\";
        public const string BackupDir=@"mwmm\backup\";
        public const string DataFile=@"mwmm\data";
        public const string SettingsFile=@"mwmm\settings2";
        public const string BSAEditFile=@"mwmm\BSAEdits";
        public const string mmodConversionData=@"mmod conversion data\";

        public static readonly string TempDir=Path.GetTempPath()+@"mwmm\";
        public static readonly string CurrentDir=(Path.GetDirectoryName(Application.ExecutablePath)+"\\").ToLower();
        public static readonly string morrowindINIDir=CurrentDir;

        public static sData Data;
        private static Mutex mutex=null;
        public static LaunchType Launch=LaunchType.None;

        [STAThread]
        public static void Main(string[] args) {
            if(!Init(args)) return;

            if(args.Length>0) {
                foreach(string s in args) LoadNewmmod(s);
            }
            //Remove any mmods that have gone missing from the mmod directory
            for(int i=0;i<Data.mmods.Count;i++) {
                if(!File.Exists(Settings.mmodDir+((mmod)Data.mmods[i]).FileName)) {
                    if(Data.mmods[i].Conflict==ConflictLevel.Active) Data.mmods[i].DeletionDeactivate();
                    Data.mmods.RemoveAt(i--);
                }
            }
            //Load any mmods manually placed in the mods directory
            foreach(string s in Directory.GetFiles(Settings.mmodDir, "*.mmod")) {
                if(!Data.DoesModExist(Path.GetFileName(s))) LoadNewmmod(Settings.mmodDir+Path.GetFileName(s));
            }
            //Put this is its own block so the GC can clean up after it
            {
                //Search for any esps the have appeared since the last time this was run
                List<string> Plugins=new List<string>();
                foreach(string s in Directory.GetFiles("data files")) {
                    if(Path.GetExtension(s)!=".esp"&&Path.GetExtension(s)!=".esm") continue;
                    if(!Data.DoesEspExist(Path.GetFileName(s))) {
                        Plugins.Add(Path.GetFileName(s));
                    }
                    FileInfo fi=new FileInfo(s);
                    if((fi.Attributes&FileAttributes.ReadOnly)>0) fi.Attributes^=FileAttributes.ReadOnly;
                }
                if(Plugins.Count>0) {
                    string[] files=Plugins.ToArray();
                    for(int i=0;i<files.Length;i++) {
                        EspInfo ei;
                        try {
                            ei=new EspInfo(files[i]);
                        } catch {
                            MessageBox.Show("File "+files[i]+" does not appear to be a valid TES4 plugin", "Error");
                            continue;
                        }
                        ei.Active=false;
                        Data.Esps.Add(ei);
                    }
                }
                //Check that the plugins we think are active are active
                Plugins.Clear();
                foreach(EspInfo ei in Data.Esps) Plugins.Add(ei.FileName);
                bool[] b=MorrowindESP.ArePluginsActive(Plugins.ToArray());
                for(int i=0;i<b.Length;i++) Data.Esps[i].Active=b[i];
                //Remove any plugins which have been deleted since the last time mwmm was run
                for(int i=0;i<Data.Esps.Count;i++) {
                    if(!File.Exists("data files\\"+Data.Esps[i].FileName)) {
                        Data.Esps.RemoveAt(i--);
                    }
                }
                //Remove any data files which have been deleted since the last time mwmm was run
                for(int i=0;i<Data.DataFiles.Count;i++) {
                    if(!File.Exists("data files\\"+Data.DataFiles[i].FileName)) {
                        foreach(mmod o in Data.mmods) {
                            if(o.Conflict==ConflictLevel.Active) {
                                int j=Array.IndexOf<DataFileInfo>(o.DataFiles, Data.DataFiles[i]);
                                if(j!=-1) {
                                    for(int k=j;k<o.DataFiles.Length-1;k++) {
                                        o.DataFiles[k]=o.DataFiles[k+1];
                                    }
                                    Array.Resize<DataFileInfo>(ref o.DataFiles, o.DataFiles.Length-1);
                                }
                            }
                        }
                        Data.DataFiles.RemoveAt(i--);
                    }
                }
                //Update any data fields which were created since the last time mwmm was run
                if(!UpdateData()) return;

            }
            //Run!
            Data.SortEsps();
            Conflicts.UpdateConflicts();
            MainForm mf=new MainForm();
            Application.Run(mf);
            Exit();
        }

        public static bool IsSafeFileName(string s) {
            s=s.Replace('/', '\\');
            if(s.IndexOfAny(Path.GetInvalidPathChars())!=-1) return false;
            if(Path.IsPathRooted(s)) return false;
            if(s.StartsWith(".")||Array.IndexOf<char>(Path.GetInvalidFileNameChars(), s[0])!=-1) return false;
            if(s.Contains("\\..\\")) return false;
            if(s.EndsWith(".")||Array.IndexOf<char>(Path.GetInvalidFileNameChars(), s[s.Length-1])!=-1) return false;
            return true;
        }

        private static bool UpdateData() {
            return true;
        }

        private static bool Init(string[] args) {
            //Initial setup
            Directory.SetCurrentDirectory(Path.GetDirectoryName(Application.ExecutablePath));
            Application.EnableVisualStyles();
            AppDomain.CurrentDomain.UnhandledException+=new UnhandledExceptionEventHandler(CurrentDomain_UnhandledException);
            //Check for mutex
            bool Exists=false;
            try {
                mutex=Mutex.OpenExisting("mwmm_mutex");
                Exists=true;
            } catch {
                mutex=new Mutex(true, "mwmm_mutex");
            }
            if(Exists) {
                if(args.Length>0) {
                    File.WriteAllLines("mwmm\\pipe", args);
                    return false;
                } else {
                    MessageBox.Show("Only one instance of mwmm may be open at any one time.","Error");
                    return false;
                }
            }
            //check it's being run from the morrowind directory
            if(!Directory.Exists("data files")||!File.Exists("morrowind.exe")||!File.Exists("morrowind.ini")) {
                MessageBox.Show("morrowind mod manager must be installed to morrowind's base directory.\n\n"+
                    "If you have moved or renamed 'morrowind_default.ini', please replace it.", "Error");
                return false;
            }
            //Clear out any tempory files mwmm may have created
            ClearTempFiles();
            //Create required directories
            try {
                if(!Directory.Exists("mwmm")) Directory.CreateDirectory("mwmm");
                if(!Directory.Exists(Settings.mmodDir)) Directory.CreateDirectory(Settings.mmodDir);
                if(!Directory.Exists(CorruptDir)) Directory.CreateDirectory(CorruptDir);
                if(!Directory.Exists(BackupDir)) Directory.CreateDirectory(BackupDir);
                File.Delete("mwmm\\pipe");
            } catch (Exception ex) {
                MessageBox.Show("One or more of morrowind mod managers directories do not exist and cannot be created.\n"+
                    "Error: "+ex.Message, "Error");
                return false;
            }
            if(!MorrowindINI.CreateINI()) {
                MessageBox.Show("morrowind's ini file is missing and could not be created.", "Error");
                return false;
            }
            //Check secutity
            try {
                File.Delete("mwmm\\limited");
                FileStream fs=File.Create("mwmm\\limited");
                fs.Close();
                File.Delete("mwmm\\limited");
            } catch {
                MessageBox.Show("Morrowind mod manager requires write access to morrowinds base folder.", "Error");
                return false;
            }
            //Load saved settings
            MorrowindBSA.ReadArchives();
            Settings.LoadSettings();
            if(!File.Exists(DataFile)) {
                Data=new sData();
            } else {
                Formatter f=new Formatter();
                Stream s=File.OpenRead(DataFile);
                try {
                    Data=(sData)f.Deserialize(s);
                } catch(Exception ex) {
                    MessageBox.Show("Error loading saved data.\n"+
                        "If you are upgrading from mwmm 0.7.x all mwmm activation information will be lost\n"+
                        "If you did not deactivate all mmods before upgrading, please use the 'clean all' button before starting to reactivate them.\n"+
                        "Error: "+ex.Message, "Error");
                    Data=new sData();
                }
                s.Close();
            }
            //recreate the unserialized data in mmods
            foreach(mmod o in Data.mmods) {
                o.RecreatePrivateData();
            }
            //finished sucessfully
            return true;
        }

        static void CurrentDomain_UnhandledException(object sender, UnhandledExceptionEventArgs e) {
            string s="An unhandled exception occurred, and probably resulted from a bug in morrowind mod manager."+
                Environment.NewLine+
                "Please post/PM me the contents of this error on the TES official forums, and I'll fix it asap."+
                Environment.NewLine+
                "The information below should also be saved to 'mwmm_crashdump.txt' in morrowind's base directory for easy copy/pasting."+
                Environment.NewLine+Environment.NewLine+
                "Error message: "+((Exception)e.ExceptionObject).Message+
                Environment.NewLine+
                "Stack trace: "+((Exception)e.ExceptionObject).StackTrace;
            MessageBox.Show(s, "Fatal error");
            try {
                StreamWriter sw=new StreamWriter("mwmm_crashdump.txt");
                sw.WriteLine(s);
                sw.Close();
            } catch { }
        }

        private static void Exit() {
            ClearTempFiles();
            if(Settings.LockFOV) {
                try {
                    MorrowindINI.WriteINIValue("[Display]", "fDefaultFOV", "75.0000");
                } catch (UnauthorizedAccessException) {
                    MessageBox.Show("Unable to set FOV to 75\n"+
                        "You appear to have morrowind.ini write protected\n"+
                        "The lock FOV option has been disabled, and can be reenabled from the options menu", "Error");
                    Settings.LockFOV=false;
                }
            }
            MorrowindESP.SetActivePlugins();
                try {
                    MorrowindBSA.CommitArchives();
                } catch(UnauthorizedAccessException) {
                    MessageBox.Show("mwmm was unable to commit the BSA list to morrowind.ini\n"+
                    "You appear to have morrowind.ini write protected\n"+
                    "The BSA list will now be incorrect, and some mods may not work as intended", "Error");
                }
            
            //Save settings, if not running as a limited user
                Settings.SaveSettings();
                Stream s=File.Open(DataFile, FileMode.Create);
                Formatter f=new Formatter();
                f.Serialize(s, Data);
                s.Close();
            mutex.Close();
            //run morrowind!
            switch(Launch) {
            case LaunchType.morrowind:
                try {
                    if(Settings.morrowindCommandLine!="") {
                        System.Diagnostics.Process.Start(Settings.morrowindCommandLine);
                    } else {
                        System.Diagnostics.Process.Start("morrowind.exe");
                    }
                } catch(Exception ex) {
                    MessageBox.Show("An error occurred attempting to start morrowind.\n"+
                    ex.Message, "Error");
                }
                break;
            case LaunchType.Updater:
                try {
                    System.Diagnostics.ProcessStartInfo psi=new System.Diagnostics.ProcessStartInfo(CurrentDir+"mwmm\\mwmmUpdater.exe");
                    psi.WorkingDirectory=CurrentDir+"mwmm\\";
                    System.Diagnostics.Process.Start(psi);
                } catch(Exception ex) {
                    MessageBox.Show("An error occurred attempting to apply the update.\n"+
                    ex.Message, "Error");
                }
                break;
            }
        }

        public static bool strArrayContains(List<string> a, string s) {
            s=s.ToLower();
            foreach(string s2 in a) {
                if(s2.ToLower()==s) return true;
            }
            return false;
        }
        public static bool strArrayContains(List<DataFileInfo> a, string s) {
            s=s.ToLower();
            for(int i=0;i<a.Count;i++) {
                if(a[i].LowerFileName==s) return true;
            }
            return false;
        }

        public static DataFileInfo strArrayGet(DataFileInfo[] a, string s) {
            s=s.ToLower();
            for(int i=0;i<a.Length;i++) {
                if(a[i].LowerFileName==s) return a[i];
            }
            return null;
        }

        public static void strArrayRemove(List<string> a, string s) {
            s=s.ToLower();
            for(int i=0;i<a.Count;i++) {
                if(a[i].ToLower()==s) {
                    a.RemoveAt(i);
                    return;
                }
            }
        }
        public static void strArrayRemove(List<DataFileInfo> a, string s) {
            s=s.ToLower();
            for(int i=0;i<a.Count;i++) {
                if(a[i].LowerFileName==s) {
                    a.RemoveAt(i);
                    return;
                }
            }
        }

        public static mmod LoadNewmmod(string s) {
            //Perform some sanity checking before trying to load the mmod
            if(!File.Exists(s)) {
                MessageBox.Show("Unable to find file '"+s+"'", "Error");
                return null;
            }
            if(Path.GetExtension(s)!=".mmod") {
                MessageBox.Show("Unrecognized file type. morrowind mod manager only supports '.mmod' "+
                                "files", "Error");
                return null;
            }
            if(Path.GetDirectoryName(s).ToLower()+"\\"==CurrentDir+Settings.mmodDir) {
                return null;
            }
            if(Data.DoesModExist(Path.GetFileName(s))) {
                MessageBox.Show("The mod '"+Path.GetFileName(s)+"' appears to already be installed.",
                    "Error");
                return null;
            }
            if(s.ToLower()==Settings.mmodDir+Path.GetFileName(s).ToLower()) {
                //Register a newly created mmod
                mmod newmod=null;
                try {
                    newmod=new mmod(Path.GetFileName(s),true);
                    FileInfo fi=new FileInfo(Settings.mmodDir+Path.GetFileName(s));
                    fi.CreationTime=DateTime.Now;
                    Data.mmods.Add(newmod);
                    Conflicts.UpdateConflict(newmod);
                    return newmod;
                } catch(Exception ex) {
                    if(newmod!=null) newmod.Close();
                    MessageBox.Show("Error loading '"+s+"'\n"+ex.Message, "Error");
                    try { File.Move(s, CorruptDir+Path.GetFileName(s)); } catch { }
                    return null;
                }
            } else {
                //Register a newly downloaded mmod
                try {
                    if(Settings.ShowNewModInfo) {
                        if(MessageBox.Show("Show mmod info for '"+s+"'?", "Question", System.Windows.Forms.MessageBoxButtons.YesNo)==System.Windows.Forms.DialogResult.Yes) {
                            (new TextEditor(s, new mmod(s,false).GetInfo(), false, false)).ShowDialog();
                        }
                        if(MessageBox.Show("Install '"+s+"'?", "Question", System.Windows.Forms.MessageBoxButtons.YesNo)!=System.Windows.Forms.DialogResult.Yes) {
                            return null;
                        }
                    }
                    File.Copy(s, Settings.mmodDir+Path.GetFileName(s));
                    mmod newmod=new mmod(Path.GetFileName(s),true);
                    
                    newmod.Close();
                    FileInfo fi=new FileInfo(Settings.mmodDir+Path.GetFileName(s));
                    if((fi.Attributes&FileAttributes.ReadOnly)>0) fi.Attributes^=FileAttributes.ReadOnly;
                    fi.CreationTime=DateTime.Now;
                    Data.mmods.Add(newmod);
                    Conflicts.UpdateConflict(newmod);
                    return newmod;
                } catch(Exception ex) {
                    MessageBox.Show("Error loading '"+s+"'\n"+ex.Message, "Error");
                    try { File.Delete(Settings.mmodDir+Path.GetFileName(s)); } catch { }
                    return null;
                }
            }
        }

        public static FileStream CreateTempFile() {
            string s;
            return CreateTempFile(out s);
        }
        public static FileStream CreateTempFile(out string path) {
            int i=0;
            for(i=0;i<32000;i++) {
                if(!File.Exists(TempDir+"tmp"+i.ToString())) {
                    path=TempDir+"tmp"+i.ToString();
                    return File.Create(path);
                }
            }
            throw new mwmmException("Could not create temp file because directory is full");
        }

        public static string CreateTempDirectory() {
            for(int i=0;i<32000;i++) {
                if(!Directory.Exists(TempDir+i.ToString())) {
                    Directory.CreateDirectory(TempDir+i.ToString()+"\\");
                    return TempDir+i.ToString()+"\\";
                }
            }
            throw new mwmmException("Could not create temp folder because directory is full");
        }

        public static void ClearTempFiles() { ClearTempFiles(""); }
        public static void ClearTempFiles(string subfolder) {
            if(!Directory.Exists(TempDir)) Directory.CreateDirectory(TempDir);
            if(!Directory.Exists(TempDir+subfolder)) return;
            foreach(string file in Directory.GetFiles(TempDir+subfolder)) {
                try { File.Delete(file); } catch { }
            }
            try { Directory.Delete(TempDir+subfolder, true); } catch { }
            if(!Directory.Exists(TempDir)) Directory.CreateDirectory(TempDir);
        }
        
        public static string ReadAllText(string file) {
        	if(!File.Exists(file)) return null;
        	return File.ReadAllText(file, System.Text.Encoding.Default);
        }

        public static string ReadCString(BinaryReader br) {
            string s="";
            while(true) {
                byte b=br.ReadByte();
                if(b==0) return s;
                s+=(char)b;
            }
        }
    }
}
